#!/usr/bin/env python# -*- coding:utf-8 -*-

"""
Date: 2019/11/30 13:14
Desc: 网站的经济政策不确定性指数
http://www.policyuncertainty.com/index.html
"""
import pandas as pd


def article_epu_index(index: str = "China") -> pd.DataFrame:
    """
    网站的经济政策不确定性指数
    :param index: 指定的国家名称, e.g. “China”
    :type index: str
    :return: 指定 index 的数据
    :rtype: pandas.DataFrame
    """
    if index == "China New":
        index = "China"
    if index == "USA":
        index = "US"
    if index == "Hong Kong":
        index = "HK"
        epu_df = pd.read_excel(
            f"http://www.policyuncertainty.com/media/{index}_EPU_Data_Annotated.xlsx",
            engine="openpyxl"
        )
        return epu_df
    if index in ["Germany", "France", "Italy"]:  # 欧洲
        index = "Europe"
    if index == "South Korea":
        index = "Korea"
    if index == "Spain New":
        index = "Spain"
    if index in ["Ireland", "Chile", "Colombia", "Netherlands", "Singapore", "Sweden"]:
        epu_df = pd.read_excel(
            f"http://www.policyuncertainty.com/media/{index}_Policy_Uncertainty_Data.xlsx",
            engine="openpyxl"
        )
        return epu_df
    if index == "Greece":
        epu_df = pd.read_excel(
            f"http://www.policyuncertainty.com/media/FKT_{index}_Policy_Uncertainty_Data.xlsx",
            engine="openpyxl"
        )
        return epu_df
    url = f"http://www.policyuncertainty.com/media/{index}_Policy_Uncertainty_Data.csv"
    epu_df = pd.read_csv(url)
    return epu_df


if __name__ == "__main__":
    article_epu_index_df = article_epu_index(index="Greece")
    print(article_epu_index_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/11/25 17:23
Desc: 期货日线行情
"""
import datetime
import json
import re
import zipfile
from io import BytesIO, StringIO

import numpy as np
import pandas as pd
import requests

from mssdk.futures import cons
from mssdk.futures.requests_fun import requests_link

calendar = cons.get_calendar()


def get_cffex_daily(date: str = "20100416") -> pd.DataFrame:
    """
    中国金融期货交易所-日频率交易数据
    http://www.cffex.com.cn/rtj/
    :param date: 交易日; 数据开始时间为 20100416
    :type date: str
    :return: 日频率交易数据
    :rtype: pandas.DataFrame
    """
    day = cons.convert_date(date) if date is not None else datetime.date.today()
    if day.strftime("%Y%m%d") not in calendar:
        # warnings.warn("%s非交易日" % day.strftime("%Y%m%d"))
        return None
    url = f"http://www.cffex.com.cn/sj/historysj/{date[:-2]}/zip/{date[:-2]}.zip"
    r = requests.get(url)
    try:
        with zipfile.ZipFile(BytesIO(r.content)) as file:
            with file.open(f"{date}_1.csv") as my_file:
                data = my_file.read().decode("gb2312")
                data_df = pd.read_csv(StringIO(data))
    except:
        return None
    data_df = data_df[data_df["合约代码"] != "小计"]
    data_df = data_df[data_df["合约代码"] != "合计"]
    data_df = data_df[~data_df["合约代码"].str.contains("IO")]
    data_df.reset_index(inplace=True, drop=True)
    data_df["合约代码"] = data_df["合约代码"].str.strip()
    symbol_list = data_df["合约代码"].to_list()
    variety_list = [re.compile(r"[a-zA-Z_]+").findall(item)[0] for item in symbol_list]
    if data_df.shape[1] == 15:
        data_df.columns = [
            "symbol",
            "open",
            "high",
            "low",
            "volume",
            "turnover",
            "open_interest",
            "_",
            "close",
            "settle",
            "pre_settle",
            "_",
            "_",
            "_",
            "_",
        ]
    else:
        data_df.columns = [
            "symbol",
            "open",
            "high",
            "low",
            "volume",
            "turnover",
            "open_interest",
            "_",
            "close",
            "settle",
            "pre_settle",
            "_",
            "_",
            "_",
        ]
    data_df["date"] = date
    data_df["variety"] = variety_list
    data_df = data_df[
        [
            "symbol",
            "date",
            "open",
            "high",
            "low",
            "close",
            "volume",
            "open_interest",
            "turnover",
            "settle",
            "pre_settle",
            "variety",
        ]
    ]
    return data_df


def get_ine_daily(date: str = "20200106") -> pd.DataFrame:
    """
    上海国际能源交易中心-日频率-量价数据
    上海国际能源交易中心: 原油期货(上市时间: 20180326); 20号胶期货(上市时间: 20190812)
    trade_price: http://www.ine.cn/statements/daily/?paramid=kx
    trade_note: http://www.ine.cn/data/datanote.dat
    :param date: 日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象，默认为当前交易日
    :type date: str or datetime.date
    :return: 上海国际能源交易中心-日频率-量价数据
    :rtype: pandas.DataFrame or None
    """
    day = cons.convert_date(date) if date is not None else datetime.date.today()
    if day.strftime("%Y%m%d") not in calendar:
        # warnings.warn(f"{day.strftime('%Y%m%d')}非交易日")
        return None
    url = f"http://www.ine.cn/data/dailydata/kx/kx{day.strftime('%Y%m%d')}.dat"
    r = requests.get(url)
    result_df = pd.DataFrame()
    try:
        data_json = r.json()
    except:
        return None
    temp_df = pd.DataFrame(data_json["o_curinstrument"]).iloc[:-1, :]
    temp_df = temp_df[temp_df["DELIVERYMONTH"] != "小计"]
    temp_df = temp_df[~temp_df["PRODUCTNAME"].str.contains("总计")]
    try:
        result_df["symbol"] = (
            temp_df["PRODUCTGROUPID"].str.upper().str.strip() + temp_df["DELIVERYMONTH"]
        )
    except:
        result_df["symbol"] = (
            temp_df["PRODUCTID"]
            .str.upper()
            .str.strip()
            .str.split("_", expand=True)
            .iloc[:, 0]
            + temp_df["DELIVERYMONTH"]
        )
    result_df["date"] = day.strftime("%Y%m%d")
    result_df["open"] = temp_df["OPENPRICE"]
    result_df["high"] = temp_df["HIGHESTPRICE"]
    result_df["low"] = temp_df["LOWESTPRICE"]
    result_df["close"] = temp_df["CLOSEPRICE"]
    result_df["volume"] = temp_df["VOLUME"]
    result_df["open_interest"] = temp_df["OPENINTEREST"]
    result_df["turnover"] = 0
    result_df["settle"] = temp_df["SETTLEMENTPRICE"]
    result_df["pre_settle"] = temp_df["PRESETTLEMENTPRICE"]
    try:
        result_df["variety"] = temp_df["PRODUCTGROUPID"].str.upper().str.strip()
    except:
        result_df["variety"] = (
            temp_df["PRODUCTID"]
            .str.upper()
            .str.strip()
            .str.split("_", expand=True)
            .iloc[:, 0]
        )
    result_df = result_df[result_df["symbol"] != "总计"]
    result_df = result_df[~result_df["symbol"].str.contains("efp")]
    return result_df


def get_czce_daily(date: str = "20050525") -> pd.DataFrame:
    """
    郑州商品交易所-日频率-量价数据
    :param date: 日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象，默认为当前交易日; 日期需要大于 20100824
    :type date: str or datetime.date
    :return: 郑州商品交易所-日频率-量价数据
    :rtype: pandas.DataFrame or None
    """
    day = cons.convert_date(date) if date is not None else datetime.date.today()
    if day.strftime("%Y%m%d") not in calendar:
        # warnings.warn(f"{day.strftime('%Y%m%d')}非交易日")
        return None
    if day > datetime.date(2010, 8, 24):
        if day > datetime.date(2015, 11, 11):
            u = cons.CZCE_DAILY_URL_3
            url = u % (day.strftime("%Y"), day.strftime("%Y%m%d"))
        elif day <= datetime.date(2015, 11, 11):
            u = cons.CZCE_DAILY_URL_2
            url = u % (day.strftime("%Y"), day.strftime("%Y%m%d"))
        listed_columns = cons.CZCE_COLUMNS
        output_columns = cons.OUTPUT_COLUMNS
        try:
            r = requests.get(url)
            if datetime.date(2015, 11, 12) <= day <= datetime.date(2017, 12, 27):
                html = str(r.content, encoding="gbk")
            else:
                html = r.text
        except requests.exceptions.HTTPError as reason:
            if reason.response.status_code != 404:
                print(
                    cons.CZCE_DAILY_URL_3
                    % (day.strftime("%Y"), day.strftime("%Y%m%d")),
                    reason,
                )
            return None
        if html.find("您的访问出错了") >= 0 or html.find("无期权每日行情交易记录") >= 0:
            return None
        html = [
            i.replace(" ", "").split("|")
            for i in html.split("\n")[:-4]
            if i[0][0] != "小"
        ]

        if day > datetime.date(2015, 11, 11):
            if html[1][0] not in ["品种月份", "品种代码", "合约代码"]:
                return
            dict_data = list()
            day_const = int(day.strftime("%Y%m%d"))
            for row in html[2:]:
                m = cons.FUTURES_SYMBOL_PATTERN.match(row[0])
                if not m:
                    continue
                row_dict = {"date": day_const, "symbol": row[0], "variety": m.group(1)}
                for i, field in enumerate(listed_columns):
                    if row[i + 1] == "\r" or row[i + 1] == "":
                        row_dict[field] = 0.0
                    elif field in [
                        "volume",
                        "open_interest",
                        "oi_chg",
                        "exercise_volume",
                    ]:
                        row[i + 1] = row[i + 1].replace(",", "")
                        row_dict[field] = int(row[i + 1])
                    else:
                        row[i + 1] = row[i + 1].replace(",", "")
                        row_dict[field] = float(row[i + 1])
                dict_data.append(row_dict)

            return pd.DataFrame(dict_data)[output_columns]
        elif day <= datetime.date(2015, 11, 11):
            dict_data = list()
            day_const = int(day.strftime("%Y%m%d"))
            for row in html[1:]:
                row = row[0].split(",")
                m = cons.FUTURES_SYMBOL_PATTERN.match(row[0])
                if not m:
                    continue
                row_dict = {"date": day_const, "symbol": row[0], "variety": m.group(1)}
                for i, field in enumerate(listed_columns):
                    if row[i + 1] == "\r":
                        row_dict[field] = 0.0
                    elif field in [
                        "volume",
                        "open_interest",
                        "oi_chg",
                        "exercise_volume",
                    ]:
                        row_dict[field] = int(float(row[i + 1]))
                    else:
                        row_dict[field] = float(row[i + 1])
                dict_data.append(row_dict)
            return pd.DataFrame(dict_data)[output_columns]

    if day <= datetime.date(2010, 8, 24):
        u = cons.CZCE_DAILY_URL_1
        url = u % day.strftime("%Y%m%d")
        listed_columns = cons.CZCE_COLUMNS_2
        output_columns = cons.OUTPUT_COLUMNS
        df = pd.read_html(url)[1].dropna(how="any")
        dict_data = list()
        day_const = int(day.strftime("%Y%m%d"))
        for row in df.to_dict(orient="records"):
            row = list(row.values())
            m = cons.FUTURES_SYMBOL_PATTERN.match(row[0])
            if not m:
                continue
            row_dict = {"date": day_const, "symbol": row[0], "variety": m.group(1)}
            for i, field in enumerate(listed_columns):
                if row[i + 1] == "\r":
                    row_dict[field] = 0.0
                elif field in ["volume", "open_interest", "oi_chg", "exercise_volume"]:
                    row_dict[field] = int(row[i + 1])
                else:
                    row_dict[field] = float(row[i + 1])
            dict_data.append(row_dict)
        return pd.DataFrame(dict_data)[output_columns]


def get_shfe_v_wap(date: str = "20131017") -> pd.DataFrame:
    """
    上期所日成交均价数据
    Parameters
    ------
    date: 日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
    Return
    -------
    DataFrame
        郑商所日交易数据(DataFrame):
            symbol        合约代码
            date          日期
            time_range    v_wap时段，分09:00-10:15和09:00-15:00两类
            v_wap          加权平均成交均价
    或 None(给定日期没有数据)
    """
    day = cons.convert_date(date) if date is not None else datetime.date.today()
    if day.strftime("%Y%m%d") not in calendar:
        # warnings.warn("%s非交易日" % day.strftime("%Y%m%d"))
        return None
    try:
        json_data = json.loads(
            requests_link(
                cons.SHFE_V_WAP_URL % (day.strftime("%Y%m%d")),
                headers=cons.headers,
                encoding="utf-8",
            ).text
        )
    except:
        return None

    if len(json_data["o_currefprice"]) == 0:
        return None
    try:
        df = pd.DataFrame(json_data["o_currefprice"])
        df["INSTRUMENTID"] = df["INSTRUMENTID"].str.strip()
        df[":B1"].astype("int16")
        return df.rename(columns=cons.SHFE_V_WAP_COLUMNS)[
            list(cons.SHFE_V_WAP_COLUMNS.values())
        ]
    except:
        return None


def get_shfe_daily(date: str = "20160104") -> pd.DataFrame:
    """
    上海期货交易所-日频率-量价数据
    http://www.shfe.com.cn/statements/dataview.html?paramid=kx
    :param date: 日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象, 默认为当前交易日
    :type date: str or datetime.date
    :return: 上海期货交易所-日频率-量价数据
    :rtype: pandas.DataFrame or None
    上期所日交易数据(DataFrame):
    symbol        合约代码
    date          日期
    open          开盘价
    high          最高价
    low           最低价
    close         收盘价
    volume        成交量
    open_interest 持仓量
    turnover      成交额
    settle        结算价
    pre_settle     前结算价
    variety       合约类别
    或 None(给定交易日没有交易数据)
    """
    day = cons.convert_date(date) if date is not None else datetime.date.today()
    if day.strftime("%Y%m%d") not in calendar:
        # warnings.warn("%s非交易日" % day.strftime("%Y%m%d"))
        return None
    try:
        json_data = json.loads(
            requests_link(
                cons.SHFE_DAILY_URL % (day.strftime("%Y%m%d")),
                headers=cons.shfe_headers,
            ).text
        )
    except requests.HTTPError as reason:
        if reason.response != 404:
            print(cons.SHFE_DAILY_URL % (day.strftime("%Y%m%d")), reason)
        return

    if len(json_data["o_curinstrument"]) == 0:
        return

    df = pd.DataFrame(
        [
            row
            for row in json_data["o_curinstrument"]
            if row["DELIVERYMONTH"] not in ["小计", "合计"] and row["DELIVERYMONTH"] != ""
        ]
    )
    try:
        df["variety"] = df["PRODUCTGROUPID"].str.upper().str.strip()
    except KeyError as e:
        df["variety"] = (
            df["PRODUCTID"]
            .str.upper()
            .str.split("_", expand=True)
            .iloc[:, 0]
            .str.strip()
        )
    df["symbol"] = df["variety"] + df["DELIVERYMONTH"]
    df["date"] = day.strftime("%Y%m%d")
    v_wap_df = get_shfe_v_wap(day)
    if v_wap_df is not None:
        df = pd.merge(
            df,
            v_wap_df[v_wap_df.time_range == "9:00-15:00"],
            on=["date", "symbol"],
            how="left",
        )
        df["turnover"] = df.v_wap * df.VOLUME
    else:
        df["VOLUME"] = df["VOLUME"].apply(lambda x: 0 if x == "" else x)
        df["turnover"] = df["VOLUME"] * df["SETTLEMENTPRICE"]
    df.rename(columns=cons.SHFE_COLUMNS, inplace=True)
    df = df[~df["symbol"].str.contains("efp")]
    return df[cons.OUTPUT_COLUMNS]


def get_dce_daily(date: str = "20030115") -> pd.DataFrame:
    """
    大连商品交易所日交易数据
    http://www.dce.com.cn/dalianshangpin/xqsj/tjsj26/rtj/rxq/index.html
    :param date: 交易日, e.g., 20200416
    :type date: str
    :return: 具体交易日的个品种行情数据
    :rtype: pandas.DataFrame
    """
    day = cons.convert_date(date) if date is not None else datetime.date.today()
    if day.strftime("%Y%m%d") not in calendar:
        # warnings.warn("%s非交易日" % day.strftime("%Y%m%d"))
        return None
    url = "http://www.dce.com.cn/publicweb/quotesdata/exportDayQuotesChData.html"
    headers = {
        "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Content-Length": "86",
        "Content-Type": "application/x-www-form-urlencoded",
        "Host": "www.dce.com.cn",
        "Origin": "http://www.dce.com.cn",
        "Pragma": "no-cache",
        "Referer": "http://www.dce.com.cn/publicweb/quotesdata/dayQuotesCh.html",
        "Upgrade-Insecure-Requests": "1",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.105 Safari/537.36",
    }
    params = {
        "dayQuotes.variety": "all",
        "dayQuotes.trade_type": "0",
        "year": date[:4],
        "month": str(int(date[4:6]) - 1),
        "day": date[6:],
        "exportFlag": "excel",
    }
    r = requests.post(url, data=params, headers=headers)
    data_df = pd.read_excel(BytesIO(r.content))
    data_df = data_df[~data_df["商品名称"].str.contains("小计")]
    data_df = data_df[~data_df["商品名称"].str.contains("总计")]
    data_df["variety"] = data_df["商品名称"].map(lambda x: cons.DCE_MAP[x])
    data_df["symbol"] = data_df["variety"] + data_df["交割月份"].astype(int).astype(str)
    del data_df["商品名称"]
    del data_df["交割月份"]
    data_df.columns = [
        "open",
        "high",
        "low",
        "close",
        "pre_settle",
        "settle",
        "_",
        "_",
        "volume",
        "open_interest",
        "_",
        "turnover",
        "variety",
        "symbol",
    ]
    data_df["date"] = date
    data_df = data_df[
        [
            "symbol",
            "date",
            "open",
            "high",
            "low",
            "close",
            "volume",
            "open_interest",
            "turnover",
            "settle",
            "pre_settle",
            "variety",
        ]
    ]
    data_df = data_df.applymap(lambda x: x.replace(",", ""))
    data_df = data_df.astype(
        {
            "open": "float",
            "high": "float",
            "low": "float",
            "close": "float",
            "volume": "float",
            "open_interest": "float",
            "turnover": "float",
            "settle": "float",
            "pre_settle": "float",
        }
    )
    return data_df


def get_futures_daily(
    start_date: str = "20201031",
    end_date: str = "20201031",
    market: str = "CZCE",
    index_bar: bool = False,
) -> pd.DataFrame:
    """
    交易所日交易数据
    :param start_date: 开始日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
    :type start_date: str
    :param end_date: 结束数据 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
    :type end_date: str
    :param market: 'CFFEX' 中金所, 'CZCE' 郑商所,  'SHFE' 上期所, 'DCE' 大商所 之一, 'INE' 上海国际能源交易中心。默认为中金所
    :type market: str
    :param index_bar: 是否合成指数K线, 默认为 False 否则影响 roll_yield 的计算
    :type index_bar: bool
    :return: 交易所日交易数据
    :rtype: pandas.DataFrame
    """
    if market.upper() == "CFFEX":
        f = get_cffex_daily
    elif market.upper() == "CZCE":
        f = get_czce_daily
    elif market.upper() == "SHFE":
        f = get_shfe_daily
    elif market.upper() == "DCE":
        f = get_dce_daily
    elif market.upper() == "INE":
        f = get_ine_daily
    else:
        print("Invalid Market Symbol")
        return None

    start_date = (
        cons.convert_date(start_date)
        if start_date is not None
        else datetime.date.today()
    )
    end_date = (
        cons.convert_date(end_date)
        if end_date is not None
        else cons.convert_date(cons.get_latest_data_date(datetime.datetime.now()))
    )

    df_list = list()
    while start_date <= end_date:
        df = f(date=str(start_date).replace("-", ""))
        if df is not None:
            df_list.append(df)
            if index_bar:
                df_list.append(get_futures_index(df))
        start_date += datetime.timedelta(days=1)

    if len(df_list) > 0:
        temp_df = pd.concat(df_list).reset_index(drop=True)
        temp_df = temp_df[~temp_df["symbol"].str.contains("efp")]
        return temp_df


def get_futures_index(df: pd.DataFrame) -> pd.DataFrame:
    """
    指数日交易数据, 指数合成
    :param df: 爬到的原始合约日线行情
    :type df: pandas.DataFrame
    :return: 持仓量加权指数日线行情
    :rtype: pandas.DataFrame
    """
    index_dfs = []
    for var in set(df["variety"]):
        df_cut = df[df["variety"] == var]
        df_cut = df_cut[df_cut["open_interest"] != 0]
        df_cut = df_cut[df_cut["close"] != np.nan]
        df_cut = df_cut[df_cut["volume"] != int(0)]
        if len(df_cut.index) > 0:
            index_df = pd.Series(index=df_cut.columns, dtype="object")
            index_df[["volume", "open_interest", "turnover"]] = df_cut[
                ["volume", "open_interest", "turnover"]
            ].sum()
            if "efp" in df_cut.iloc[-1, 0]:
                df_cut = df_cut.iloc[:-1, :]
            df_cut.replace("", 0, inplace=True)  # 20201026 部分数据开盘价空缺
            index_df[["open", "high", "low", "close", "settle", "pre_settle"]] = (
                np.dot(
                    np.array(
                        df_cut[["open", "high", "low", "close", "settle", "pre_settle"]]
                    ).T,
                    np.array((df_cut["open_interest"].astype(float))),
                )
                / np.sum(df_cut["open_interest"].astype(float))
            )
            index_df[["date", "variety"]] = df_cut[["date", "variety"]].iloc[0, :]
            index_df["symbol"] = index_df["variety"] + "99"
            index_dfs.append(index_df)
    return pd.concat(index_dfs, axis=1).T


if __name__ == "__main__":
    get_futures_daily_df = get_futures_daily(
        start_date="20200105", end_date="20200201", market="INE", index_bar=False
    )
    print(get_futures_daily_df)

    get_dce_daily_df = get_dce_daily(date="20210427")
    print(get_dce_daily_df)

    get_cffex_daily_df = get_cffex_daily(date="20210719")
    print(get_cffex_daily_df)

    get_ine_daily_df = get_ine_daily(date="20210426")
    print(get_ine_daily_df)

    get_czce_daily_df = get_czce_daily(date="20151111")
    print(get_czce_daily_df)

    get_shfe_daily_df = get_shfe_daily(date="20160104")
    print(get_shfe_daily_df)



# -*- coding:utf-8 -*-
# /usr/bin/env python
"""
Date: 2019/12/6 0:17
Desc: 交易法门-数据

交易法门-数据-黑色系
交易法门-数据-黑色系-焦煤
交易法门-数据-黑色系-焦炭

交易法门-数据-农产品
交易法门-数据-农产品-美豆
交易法门-数据-农产品-豆粕
交易法门-数据-农产品-豆油
交易法门-数据-农产品-棕榈
交易法门-数据-农产品-白糖
"""
import requests
import pandas as pd

from mssdk.futures_derivative.cons import (
    # 交易法门-数据-农产品
    jyfm_data_usa_bean_name_url_dict,  # 交易法门-数据-农产品-美豆
    jyfm_data_soybean_meal_name_url_dict,  # 交易法门-数据-农产品-豆粕
    jyfm_data_soybean_oil_name_url_dict,  # 交易法门-数据-农产品-豆油
    jyfm_data_palm_name_url_dict,  # 交易法门-数据-农产品-棕榈
    jyfm_data_sugar_name_url_dict,  # 交易法门-数据-农产品-白糖
    # 交易法门-数据-黑色系
    jyfm_data_cocking_coal_url_dict,  # 交易法门-数据-黑色系-焦煤
    jyfm_data_coke_url_dict,  # 交易法门-数据-黑色系-焦炭
)
from mssdk.futures_derivative.jyfm_login_func import jyfm_login


# 农产品
def jyfm_data_usa_bean(indicator="美豆种植进度", headers=""):
    """
    交易法门-数据-农产品-美豆
    :param indicator: ["美豆种植进度", "美豆出苗率", "美豆开花率", "美豆优良率", "美豆收割进度", "美豆出口情况"]
    :type indicator: str
    :param headers: headers with cookies
    :type headers: dict
    :return: result
    :rtype: pandas.DataFrame
    """
    res = requests.get(jyfm_data_usa_bean_name_url_dict[indicator], headers=headers,)
    return pd.DataFrame(res.json())


def jyfm_data_soybean_meal(indicator="豆粕月度产量", headers=""):
    """
    交易法门-数据-农产品-豆粕
    :param indicator: ["大豆月度进口", "大豆原料库存", "压榨开工率", "企业压榨利润",
    "豆粕月度产量", "豆粕每日成交", "豆粕周度库存", "豆粕平衡表"]
    :type indicator: str
    :param headers: headers with cookies
    :type headers: dict
    :return: result
    :rtype: pandas.DataFrame
    """
    res = requests.get(
        jyfm_data_soybean_meal_name_url_dict[indicator], headers=headers,
    )
    return pd.DataFrame(res.json())


def jyfm_data_soybean_oil(indicator="豆油年度产能", headers=""):
    """
    交易法门-数据-农产品-豆油
    :param indicator: ["豆油年度产能", "压榨装置开工率", "油厂周度产量", "豆油现货成交",
    "豆油商业库存", "豆油毛利润", "豆油月度产量", "豆油月度进口", "豆油月度消费", "豆油月度出口", "豆油月度库存"]
    :type indicator: str
    :param headers: headers with cookies
    :type headers: dict
    :return: result
    :rtype: pandas.DataFrame
    """
    res = requests.get(jyfm_data_soybean_oil_name_url_dict[indicator], headers=headers,)
    return pd.DataFrame(res.json())


def jyfm_data_palm(indicator="马棕种植面积", headers=""):
    """
    交易法门-数据-农产品-棕榈
    :param indicator: ["马棕种植面积", "马棕FFB单产", "马棕出油率", "马棕月度产量", "马棕月度库存", "马棕月度出口"]
    :type indicator: str
    :param headers: headers with cookies
    :type headers: dict
    :return: result
    :rtype: pandas.DataFrame
    """
    res = requests.get(jyfm_data_palm_name_url_dict[indicator], headers=headers,)
    return pd.DataFrame(res.json())


def jyfm_data_sugar(indicator="白糖产区库存", headers=""):
    """
    交易法门-数据-农产品-白糖
    :param indicator: ["国内种植面积", "年度产糖率", "白糖年度产销", "白糖进出口量",
    "食糖产需缺口", "白糖月度产量", "白糖月度销量", "白糖月度进口", "食糖工业库存", "白糖产区库存"]
    :type indicator: str
    :param headers: headers with cookies
    :type headers: dict
    :return: result
    :rtype: pandas.DataFrame
    """
    res = requests.get(jyfm_data_sugar_name_url_dict[indicator], headers=headers,)
    return pd.DataFrame(res.json())


# 黑色系
def jyfm_data_cocking_coal(indicator="焦煤总库存", headers=""):
    """
    交易法门-数据-黑色系-焦煤
    :param indicator: ["焦煤总库存", "焦煤焦企库存-焦煤焦企库存100", "焦煤焦企库存-焦煤焦企库存230", "焦煤钢厂库存", "焦煤港口库存"]
    :type indicator: str
    :param headers: headers with cookies
    :type headers: dict
    :return: result
    :rtype: pandas.DataFrame
    """
    res = requests.get(jyfm_data_cocking_coal_url_dict[indicator], headers=headers,)
    # 由于返回的数据长度不一致，先补齐再转置
    return pd.read_json(res.text, orient="index").T


def jyfm_data_coke(indicator="焦炭总库存", headers=""):
    """
    交易法门-数据-黑色系-焦炭
    :param indicator: ["焦企产能利用率-100家独立焦企产能利用率", "焦企产能利用率-230家独立焦企产能利用率",
    "焦炭日均产量-100家独立焦企焦炭日均产量", "焦炭日均产量-230家独立焦企焦炭日均产量", "焦炭总库存",
    "焦炭焦企库存-100家独立焦企焦炭库存", "焦炭焦企库存-230家独立焦企焦炭库存", "焦炭钢厂库存", "焦炭港口库存", "焦企焦化利润"]
    :type indicator: str
    :param headers: headers with cookies
    :type headers: dict
    :return: result
    :rtype: pandas.DataFrame
    """
    res = requests.get(jyfm_data_coke_url_dict[indicator], headers=headers,)
    # 由于返回的数据长度不一致，先补齐再转置
    return pd.read_json(res.text, orient="index").T


if __name__ == "__main__":
    headers = jyfm_login(account="", password="")
    # 农产品
    # 农产品-美豆
    jyfm_data_usa_bean_df = jyfm_data_usa_bean(indicator="美豆优良率", headers=headers)
    print(jyfm_data_usa_bean_df)
    # 农产品-豆粕
    jyfm_data_soybean_meal_df = jyfm_data_soybean_meal(
        indicator="豆粕月度产量", headers=headers
    )
    print(jyfm_data_soybean_meal_df)
    # 农产品-豆油
    jyfm_data_soybean_oil_df = jyfm_data_soybean_oil(
        indicator="豆油年度产能", headers=headers
    )
    print(jyfm_data_soybean_oil_df)
    # 农产品-棕榈
    jyfm_data_palm_df = jyfm_data_palm(indicator="马棕月度出口", headers=headers)
    print(jyfm_data_palm_df)
    # 农产品-白糖
    jyfm_data_sugar_df = jyfm_data_sugar(indicator="白糖产区库存", headers=headers)
    print(jyfm_data_sugar_df)

    # 黑色系
    # 黑色系-焦煤
    jyfm_data_cocking_coal_df = jyfm_data_cocking_coal(
        indicator="焦煤总库存", headers=headers
    )
    print(jyfm_data_cocking_coal_df)
    # 黑色系-焦炭
    jyfm_data_coke_df = jyfm_data_coke(indicator="焦炭总库存", headers=headers)
    print(jyfm_data_coke_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/1/29 15:11
Desc: 新浪财经-股票期权
https://stock.finance.sina.com.cn/option/quotes.html
期权-中金所-沪深300指数
https://stock.finance.sina.com.cn/futures/view/optionsCffexDP.php
期权-上交所-50ETF
期权-上交所-300ETF
https://stock.finance.sina.com.cn/option/quotes.html
"""
import json
import datetime
from typing import Dict, List, Tuple

import requests
from bs4 import BeautifulSoup
import pandas as pd


# 期权-中金所-沪深300指数
def option_sina_cffex_hs300_list() -> Dict[str, List[str]]:
    """
    新浪财经-中金所-沪深300指数-所有合约, 返回的第一个合约为主力合约
    目前新浪财经-中金所只有 沪深300指数 一个品种的数据
    :return: 中金所-沪深300指数-所有合约
    :rtype: dict
    """
    url = "https://stock.finance.sina.com.cn/futures/view/optionsCffexDP.php"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "lxml")
    symbol = soup.find(attrs={"id": "option_symbol"}).find("li").text
    temp_attr = soup.find(attrs={"id": "option_suffix"}).find_all("li")
    contract = [item.text for item in temp_attr]
    return {symbol: contract}


def option_sina_cffex_hs300_spot(contract: str = "io2104") -> pd.DataFrame:
    """
    中金所-沪深300指数-指定合约-实时行情
    https://stock.finance.sina.com.cn/futures/view/optionsCffexDP.php
    :param contract: 合约代码; 用 option_sina_cffex_hs300_list 函数查看
    :type contract: str
    :return: 中金所-沪深300指数-指定合约-看涨看跌实时行情
    :rtype: pd.DataFrame
    """
    url = "https://stock.finance.sina.com.cn/futures/api/openapi.php/OptionService.getOptionData"
    params = {
        "type": "futures",
        "product": "io",
        "exchange": "cffex",
        "pinzhong": contract,
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = json.loads(data_text[data_text.find("{") : data_text.rfind("}") + 1])
    option_call_df = pd.DataFrame(
        data_json["result"]["data"]["up"],
        columns=[
            "看涨合约_买量",
            "看涨合约_买价",
            "看涨合约_最新价",
            "看涨合约_卖价",
            "看涨合约_卖量",
            "看涨合约_持仓量",
            "看涨合约_涨跌",
            "行权价",
            "看涨合约_标识",
        ],
    )
    option_put_df = pd.DataFrame(
        data_json["result"]["data"]["down"],
        columns=[
            "看跌合约_买量",
            "看跌合约_买价",
            "看跌合约_最新价",
            "看跌合约_卖价",
            "看跌合约_卖量",
            "看跌合约_持仓量",
            "看跌合约_涨跌",
            "看跌合约_标识",
        ],
    )
    data_df = pd.concat([option_call_df, option_put_df], axis=1)
    return data_df


def option_sina_cffex_hs300_daily(contract: str = "io2103C4850") -> pd.DataFrame:
    """
    新浪财经-中金所-沪深300指数-指定合约-日频行情
    :param contract: 具体合约代码(包括看涨和看跌标识), 可以通过 option_sina_cffex_hs300_spot 中的 call_标识 获取
    :type contract: str
    :return: 日频率数据
    :rtype: pd.DataFrame
    """
    year = datetime.datetime.now().year
    month = datetime.datetime.now().month
    day = datetime.datetime.now().day
    url = f"https://stock.finance.sina.com.cn/futures/api/jsonp.php/var%20_{contract}{year}_{month}_{day}=/FutureOptionAllService.getOptionDayline"
    params = {"symbol": contract}
    r = requests.get(url, params=params)
    data_text = r.text
    data_df = pd.DataFrame(
        eval(data_text[data_text.find("[") : data_text.rfind("]") + 1])
    )
    data_df.columns = ["open", "high", "low", "close", "volume", "date"]
    return data_df


# 期权-上交所-50ETF
def option_sina_sse_list(symbol: str = "50ETF", exchange: str = "null") -> List[str]:
    """
    新浪财经-期权-上交所-50ETF-合约到期月份列表
    https://stock.finance.sina.com.cn/option/quotes.html
    :param symbol: 50ETF or 300ETF
    :type symbol: str
    :param exchange: null
    :type exchange: str
    :return: 合约到期时间
    :rtype: list
    """
    url = "http://stock.finance.sina.com.cn/futures/api/openapi.php/StockOptionService.getStockName"
    params = {"exchange": f"{exchange}", "cate": f"{symbol}"}
    r = requests.get(url, params=params)
    data_json = r.json()
    date_list = data_json["result"]["data"]["contractMonth"]
    return ["".join(i.split("-")) for i in date_list][1:]


def option_sina_sse_expire_day(
    trade_date: str = "202102", symbol: str = "50ETF", exchange: str = "null"
) -> Tuple[str, int]:
    """
    指定到期月份指定品种的剩余到期时间
    :param trade_date: 到期月份: 202002, 20203, 20206, 20209
    :type trade_date: str
    :param symbol: 50ETF or 300ETF
    :type symbol: str
    :param exchange: null
    :type exchange: str
    :return: (到期时间, 剩余时间)
    :rtype: tuple
    """
    url = "http://stock.finance.sina.com.cn/futures/api/openapi.php/StockOptionService.getRemainderDay"
    params = {
        "exchange": f"{exchange}",
        "cate": f"{symbol}",
        "date": f"{trade_date[:4]}-{trade_date[4:]}",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    data = data_json["result"]["data"]
    if int(data["remainderDays"]) < 0:
        url = "http://stock.finance.sina.com.cn/futures/api/openapi.php/StockOptionService.getRemainderDay"
        params = {
            "exchange": f"{exchange}",
            "cate": f"{'XD' + symbol}",
            "date": f"{trade_date[:4]}-{trade_date[4:]}",
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        data = data_json["result"]["data"]
    return data["expireDay"], int(data["remainderDays"])


def option_sina_sse_codes(
    trade_date: str = "202102", underlying: str = "510300"
) -> Tuple[List[str], List[str]]:
    """
    上海证券交易所所有看涨和看跌合约的代码
    :param trade_date: 期权到期月份
    :type trade_date: "202002"
    :param underlying: 标的产品代码 华夏上证 50ETF: 510050 or 华泰柏瑞沪深 300ETF: 510300
    :type underlying: str
    :return: 看涨看跌合约的代码
    :rtype: Tuple[List, List]
    """
    url_up = "".join(
        ["http://hq.sinajs.cn/list=OP_UP_", underlying, str(trade_date)[-4:]]
    )
    url_down = "".join(
        ["http://hq.sinajs.cn/list=OP_DOWN_", underlying, str(trade_date)[-4:]]
    )
    r = requests.get(url_up)
    data_text = r.text
    data_up = data_text.replace('"', ",").split(",")
    codes_up = [i[7:] for i in data_up if i.startswith("CON_OP_")]
    r = requests.get(url_down)
    data_text = r.text
    data_down = data_text.replace('"', ",").split(",")
    codes_down = [i[7:] for i in data_down if i.startswith("CON_OP_")]
    return codes_up, codes_down


def option_sina_sse_spot_price(code: str = "10002273") -> pd.DataFrame:
    """
    期权实时数据
    :param code: 期权代码
    :type code: str
    :return: 期权量价数据
    :rtype: pandas.DataFrame
    """
    url = f"http://hq.sinajs.cn/list=CON_OP_{code}"
    r = requests.get(url)
    data_text = r.text
    data_list = data_text[data_text.find('"') + 1 : data_text.rfind('"')].split(",")
    field_list = [
        "买量",
        "买价",
        "最新价",
        "卖价",
        "卖量",
        "持仓量",
        "涨幅",
        "行权价",
        "昨收价",
        "开盘价",
        "涨停价",
        "跌停价",
        "申卖价五",
        "申卖量五",
        "申卖价四",
        "申卖量四",
        "申卖价三",
        "申卖量三",
        "申卖价二",
        "申卖量二",
        "申卖价一",
        "申卖量一",
        "申买价一",
        "申买量一 ",
        "申买价二",
        "申买量二",
        "申买价三",
        "申买量三",
        "申买价四",
        "申买量四",
        "申买价五",
        "申买量五",
        "行情时间",
        "主力合约标识",
        "状态码",
        "标的证券类型",
        "标的股票",
        "期权合约简称",
        "振幅",
        "最高价",
        "最低价",
        "成交量",
        "成交额",
    ]
    data_df = pd.DataFrame(list(zip(field_list, data_list)), columns=["字段", "值"])
    return data_df


def option_sina_sse_underlying_spot_price(code: str = "sh510300") -> pd.DataFrame:
    """
    期权标的物的实时数据
    :param code: sh510050 or sh510300
    :type code: str
    :return: 期权标的物的信息
    :rtype: pandas.DataFrame
    """
    url = f"http://hq.sinajs.cn/list={code}"
    r = requests.get(url)
    data_text = r.text
    data_list = data_text[data_text.find('"') + 1 : data_text.rfind('"')].split(",")
    field_list = [
        "证券简称",
        "今日开盘价",
        "昨日收盘价",
        "最近成交价",
        "最高成交价",
        "最低成交价",
        "买入价",
        "卖出价",
        "成交数量",
        "成交金额",
        "买数量一",
        "买价位一",
        "买数量二",
        "买价位二",
        "买数量三",
        "买价位三",
        "买数量四",
        "买价位四",
        "买数量五",
        "买价位五",
        "卖数量一",
        "卖价位一",
        "卖数量二",
        "卖价位二",
        "卖数量三",
        "卖价位三",
        "卖数量四",
        "卖价位四",
        "卖数量五",
        "卖价位五",
        "行情日期",
        "行情时间",
        "停牌状态",
    ]
    data_df = pd.DataFrame(list(zip(field_list, data_list)), columns=["字段", "值"])
    return data_df


def option_sina_sse_greeks(code: str = "10003045") -> pd.DataFrame:
    """
    期权基本信息表
    :param code: 合约代码
    :type code: str
    :return: 期权基本信息表
    :rtype: pandas.DataFrame
    """
    url = f"http://hq.sinajs.cn/list=CON_SO_{code}"
    r = requests.get(url)
    data_text = r.text
    data_list = data_text[data_text.find('"') + 1 : data_text.rfind('"')].split(",")
    field_list = [
        "期权合约简称",
        "成交量",
        "Delta",
        "Gamma",
        "Theta",
        "Vega",
        "隐含波动率",
        "最高价",
        "最低价",
        "交易代码",
        "行权价",
        "最新价",
        "理论价值",
    ]
    data_df = pd.DataFrame(
        list(zip(field_list, [data_list[0]] + data_list[4:])), columns=["字段", "值"]
    )
    return data_df


def option_sina_sse_minute(code: str = "10003045") -> pd.DataFrame:
    """
    指定期权品种在当前交易日的分钟数据, 只能获取当前交易日的数据, 不能获取历史分钟数据
    https://stock.finance.sina.com.cn/option/quotes.html
    :param code: 期权代码
    :type code: str
    :return: 指定期权的当前交易日的分钟数据
    :rtype: pandas.DataFrame
    """
    url = "https://stock.finance.sina.com.cn/futures/api/openapi.php/StockOptionDaylineService.getOptionMinline"
    params = {"symbol": f"CON_OP_{code}"}
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = data_json["result"]["data"]
    data_df = pd.DataFrame(temp_df)
    data_df.columns = ["时间", "价格", "成交", "持仓", "均价", "日期"]
    return data_df


def option_sina_sse_daily(code: str = "10002273") -> pd.DataFrame:
    """
    指定期权的日频率数据
    :param code: 期权代码
    :type code: str
    :return: 指定期权的所有日频率历史数据
    :rtype: pandas.DataFrame
    """
    url = "http://stock.finance.sina.com.cn/futures/api/jsonp_v2.php//StockOptionDaylineService.getSymbolInfo"
    params = {"symbol": f"CON_OP_{code}"}
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = json.loads(data_text[data_text.find("(") + 1 : data_text.rfind(")")])
    temp_df = pd.DataFrame(data_json)
    temp_df.columns = ["日期", "开盘", "最高", "最低", "收盘", "成交量"]
    return temp_df


def option_sina_finance_minute(code: str = "10002530") -> pd.DataFrame:
    """
    指定期权的分钟频率数据
    https://stock.finance.sina.com.cn/option/quotes.html
    :param code: 期权代码
    :type code: str
    :return: 指定期权的分钟频率数据
    :rtype: pandas.DataFrame
    """
    url = "https://stock.finance.sina.com.cn/futures/api/openapi.php/StockOptionDaylineService.getFiveDayLine"
    params = {
        "symbol": f"CON_OP_{code}",
    }
    r = requests.get(url, params=params)
    data_text = r.json()
    temp_df = pd.DataFrame()
    for item in data_text["result"]["data"]:
        temp_df = temp_df.append(pd.DataFrame(item), ignore_index=True)
    temp_df.fillna(method="ffill", inplace=True)
    temp_df.columns = ["time", "price", "volume", "_", "average_price", "date"]
    temp_df = temp_df[["date", "time", "price", "average_price", "volume"]]
    return temp_df


if __name__ == "__main__":
    # 期权-中金所-沪深300指数
    option_sina_cffex_hs300_list_df = option_sina_cffex_hs300_list()
    print(option_sina_cffex_hs300_list_df)

    option_sina_cffex_hs300_spot_df = option_sina_cffex_hs300_spot(contract="io2104")
    print(option_sina_cffex_hs300_spot_df)

    option_sina_cffex_hs300_daily_df = option_sina_cffex_hs300_daily(
        contract="io2106C3600"
    )
    print(option_sina_cffex_hs300_daily_df)

    # 期权-上交所-50ETF
    option_sina_sse_list_df = option_sina_sse_list(symbol="50ETF", exchange="null")
    print(option_sina_sse_list_df)

    option_sina_sse_expire_day_df = option_sina_sse_expire_day(
        trade_date="202102", symbol="50ETF", exchange="null"
    )
    print(option_sina_sse_expire_day_df)

    up, down = option_sina_sse_codes(trade_date="202103", underlying="510300")
    print(up)
    print(down)

    option_sina_sse_spot_price_df = option_sina_sse_spot_price(code="10003045")
    print(option_sina_sse_spot_price_df)

    option_sina_sse_underlying_spot_price_df = option_sina_sse_underlying_spot_price(
        code="sh510300"
    )
    print(option_sina_sse_underlying_spot_price_df)

    option_sina_sse_greeks_df = option_sina_sse_greeks(code="10003045")
    print(option_sina_sse_greeks_df)

    option_sina_sse_minute_df = option_sina_sse_minute(code="10003045")
    print(option_sina_sse_minute_df)

    option_sina_sse_daily_df = option_sina_sse_daily(code="10003045")
    print(option_sina_sse_daily_df)

    option_sina_finance_minute_df = option_sina_finance_minute(code="10003045")
    print(option_sina_finance_minute_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2021/11/25 15:37
Desc: 中证商品指数
http://www.cscidx.com/index.html
"""
import pandas as pd
import requests


def futures_index_cscidx(symbol: str = "中证监控油脂油料期货指数") -> pd.DataFrame:
    """
    中证商品指数-商品指数-分时数据
    http://www.cscidx.com/index.html
    :param symbol: choice of {"中证监控软商品期货指数", "中证监控油脂油料期货指数", "中证监控饲料期货指数", "中证监控能化期货指数", "中证监控钢铁期货指数", "中证监控建材期货指数"}
    :type symbol: str
    :return: 商品指数-分时数据
    :rtype: pandas.DataFrame
    """
    futures_index_map = {
        '中证监控软商品期货指数': ['606008.CCI', '0'],
        '中证监控油脂油料期货指数': ['606005.CCI', '1'],
        '中证监控饲料期货指数': ['606004.CCI', '2'],
        '中证监控能化期货指数': ['606010.CCI', '3'],
        '中证监控钢铁期货指数': ['606011.CCI', '4'],
        '中证监控建材期货指数': ['606012.CCI', '5'],
    }
    url = "http://www.cscidx.com/cscidx/csciAction/loadTimeData"
    params = {
        'r': '0.08644997232349438'
    }
    payload = {
        'indexCode': futures_index_map[symbol][0],
        'indexType': futures_index_map[symbol][1],
        'pointer': 'all',
    }
    headers = {
        'Accept': 'application/json, text/javascript, */*; q=0.01',
        'Accept-Encoding': 'gzip, deflate',
        'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8',
        'Cache-Control': 'no-cache',
        'Content-Length': '44',
        'Connection': 'keep-alive',
        'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
        'Host': 'www.cscidx.com',
        'Origin': 'http://www.cscidx.com',
        'Pragma': 'no-cache',
        'Proxy-Connection': 'keep-alive',
        'Referer': 'http://www.cscidx.com/cscidx/quote1',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36',
        'X-Requested-With': 'XMLHttpRequest',
    }
    r = requests.post(url, params=params, data=payload, headers=headers)
    data_json = r.json()
    # TODO
    return data_json


if __name__ == "__main__":
    futures_index_cscidx_df = futures_index_cscidx(symbol='中证监控油脂油料期货指数')
    print(futures_index_cscidx_df)



# -*- coding:utf-8 -*-
# /usr/bin/env python
"""
Date: 2021/1/26 10:39
Desc: 宽客在线-阿尔戈斯全网监控预警系统
https://www.quantinfo.com/Argus/
此接口的目标网站稳定性较差, 可能造成部分时间段无法访问
"""
import pandas as pd
import requests
import urllib3


def watch_argus():
    """
    宽客在线-阿尔戈斯全网监控预警系统
    https://www.quantinfo.com/Argus/
    :return: 阿尔戈斯全网监控预警系统的监控数据
    :rtype: pandas.DataFrame
    """
    urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
    url = "https://www.quantinfo.com/API/Argus/predict"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.163 Safari/537.36",
    }
    r = requests.get(url, headers=headers, verify=False)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json)
    temp_df["time"] = pd.to_datetime(temp_df["time"], unit="s", utc=True).dt.tz_convert('Asia/Shanghai')
    return temp_df


if __name__ == '__main__':
    watch_argus_df = watch_argus()
    print(watch_argus_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2020/12/19 10:34
Desc: 从和讯网获取美股-中概股实时行情数据和历史行情数据(日)
http://quote.hexun.com/default.html#ustock_0
注意: 由于涉及到复权问题, 建议使用新浪的接口来获取历史行情数据(日)
"""
import json

import requests
import pandas as pd


def stock_us_zh_spot() -> pd.DataFrame:
    """
    美股-中概股的实时行情数据
    http://quote.hexun.com/default.htm#ustock_3
    :return: 中概股的实时行情数据
    :return: pandas.DataFrame
    """
    url = "http://quote.hexun.com/usastock/data/getdjstock.aspx"
    params = {
        "type": "1",
        "market": "3",
        "sorttype": "4",
        "updown": "up",
        "page": "1",
        'count': "200",
        "time": "203450"
    }
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Host": "quote.hexun.com",
        "Pragma": "no-cache",
        "Referer": "http://quote.hexun.com/usastock/xqstock.aspx?market=3",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.120 Safari/537.36"
    }
    r = requests.get(url, params=params, headers=headers)
    data_list = eval(r.text.split("=")[1].strip().rsplit(";")[0])  # eval 出列表
    data_df = pd.DataFrame(
        data_list,
        columns=[
            "代码",
            "名称",
            "最新价(美元)",
            "涨跌幅",
            "d_1",
            "d_2",
            "最高",
            "最低",
            "昨收",
            "d_3",
            "成交量",
            "d_4",
        ],
    )
    temp_df = data_df[["代码", "名称", "最新价(美元)", "涨跌幅", "最高", "最低", "昨收", "成交量"]]
    temp_df = temp_df.rename({"最新价(美元)": "最新价"}, axis=1)
    return temp_df


def stock_us_zh_daily(symbol: str = 'BABA') -> pd.DataFrame:
    """
    美股-中概股的日频率历史行情数据
    http://stockdata.stock.hexun.com/us/BABA.shtml
    :return: 中概股的日频率历史行情数据
    :return: pandas.DataFrame
    """
    url = "http://webusstock.hermes.hexun.com/usa/kline"
    params = {
        "code": f"NYSE{symbol}",
        "start": "20201218223000",
        "number": "-1000",
        "type": "5",
    }
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Cookie": "UM_distinctid=1758d5cddd49c-0fc7e2c2612624-303464-1fa400-1758d5cddd592f; ADVC=38ffe1fbf97465; cn_1263247791_dplus=%7B%22distinct_id%22%3A%20%221758d5cddd49c-0fc7e2c2612624-303464-1fa400-1758d5cddd592f%22%2C%22userFirstDate%22%3A%20%2220201103%22%2C%22userID%22%3A%20%220%22%2C%22userName%22%3A%20%22%22%2C%22userType%22%3A%20%22loginuser%22%2C%22userLoginDate%22%3A%20%2220201103%22%2C%22%24_sessionid%22%3A%200%2C%22%24_sessionTime%22%3A%201604394498%2C%22%24dp%22%3A%200%2C%22%24_sessionPVTime%22%3A%201604394498%2C%22initial_view_time%22%3A%20%221604392649%22%2C%22initial_referrer%22%3A%20%22https%3A%2F%2Fwww.baidu.com%2Flink%3Furl%3DRaFkqqESxpi2iDV4Q7Men69HaM9QOkW5KKUtQakjQzkfkygaOuGzJBFcHSg35wmfSKFA26xUDad7jHwCCv1ksa%26wd%3D%26eqid%3Da38e871500004846000000065fa11d78%22%2C%22initial_referrer_domain%22%3A%20%22www.baidu.com%22%2C%22%24recent_outside_referrer%22%3A%20%22www.baidu.com%22%7D; HexunTrack=SID=20200722150527074dded739ba5e24fd2915f1f692d4ad9c7&CITY=51&TOWN=510100; ADVS=392342a9ca40f5; ASL=18614,00rzr,abdfc0a27d461d18b68a5530; __jsluid_h=35f649169d5fb027d0e857b8ecd24d5b",
        "Host": "webusstock.hermes.hexun.com",
        "Pragma": "no-cache",
        "Referer": "http://stockdata.stock.hexun.com/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36",
        "X-Requested-With": "ShockwaveFlash/33.0.0.432",
    }
    r = requests.get(url, params=params, headers=headers)
    r.encoding = "utf-8"
    data_dict = json.loads(r.text[1:-2])
    data_df = pd.DataFrame(
        data_dict["Data"][0],
        columns=[list(item.values())[0] for item in data_dict["KLine"]],
    )
    data_df['时间'] = data_df['时间'].astype(str).str.slice(0, 8)
    data_df['前收盘价'] = round(data_df['前收盘价'] / 100, 2)
    data_df['开盘价'] = round(data_df['开盘价'] / 100, 2)
    data_df['收盘价'] = round(data_df['收盘价'] / 100, 2)
    data_df['最高价'] = round(data_df['最高价'] / 100, 2)
    data_df['最低价'] = round(data_df['最低价'] / 100, 2)
    del data_df['成交额']
    return data_df


if __name__ == "__main__":
    stock_us_zh_spot_df = stock_us_zh_spot()
    print(stock_us_zh_spot_df)
    stock_us_zh_daily_df = stock_us_zh_daily(symbol="BABA")
    print(stock_us_zh_daily_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/11/15 17:52
Desc: 乘联会
http://data.cpcaauto.com/FuelMarket

汽车行业制造企业数据库
http://i.gasgoo.com/data/ranking
"""
import requests
import pandas as pd
from mssdk.utils import demjson


def car_cpca_energy_sale():
    """
    乘联会-新能源细分市场-整体市场
    http://data.cpcaauto.com/FuelMarket
    :return: 新能源细分市场-整体市场
    :rtype: pandas.DataFrame
    """
    url = 'http://cpcadata.chinaautomarket.com:8081/chartlist'
    params = {
        'charttype': '6'
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json[0]['dataList'])
    temp_current_year_list = []
    temp_previous_year_list = []
    for item in data_json[0]['dataList']:
        temp_previous_year_list.append(item[temp_df.columns[2]])
        try:
            temp_current_year_list.append(item[temp_df.columns[1]])
        except:
            continue

    temp_current_year_df = pd.DataFrame(temp_current_year_list)
    temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
    big_df = pd.DataFrame([temp_current_year_df.iloc[:, 2], temp_previous_year_df.iloc[:, 2]]).T
    big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
    big_df["月份"] = temp_df["month"]
    big_df = big_df[[
        "月份",
        temp_df.columns[2],
        temp_df.columns[1],

    ]]
    return big_df


def car_gasgoo_sale_rank(symbol: str = "车企榜", date: str = "202109"):
    """
    盖世汽车-汽车行业制造企业数据库-销量数据
    http://i.gasgoo.com/data/ranking
    :param symbol: choice of {"车企榜", "品牌榜", "车型榜"}
    :type symbol: str
    :param date: 查询的年份和月份
    :type date: str
    :return: 销量数据
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "车型榜": "M",
        "车企榜": "F",
        "品牌榜": "B",
    }
    url = "https://i.gasgoo.com/data/sales/AutoModelSalesRank.aspx/GetSalesRank"
    payload = {
        "rankType": symbol_map[symbol],
        "startY": date[:4],
        "startM": str(int(date[4:6])),
        "endY": date[:4],
        "endM": str(int(date[4:6])),
        "orderBy": f"{date[:4]}-{str(int(date[4:6]))}",
        "modelGradeID": "",
        "modelTypeID": "",
        "countryID": "",
        "queryDate": f"{date[:4]}-{str(int(date[4:6]))}",
    }
    headers = {
        'Accept': 'application/json, text/javascript, */*; q=0.01',
        'Accept-Encoding': 'gzip, deflate, br',
        'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8',
        'Cache-Control': 'no-cache',
        'Connection': 'keep-alive',
        'Content-Length': '195',
        'Content-Type': 'application/json; charset=UTF-8',
        'Cookie': 'Hm_lvt_8e90480b1bf68ede548c407057660718=1636981448; _ga=GA1.2.858318653.1636981449; _gid=GA1.2.1448165285.1636981449; _gat=1; Hm_lpvt_8e90480b1bf68ede548c407057660718=1636982578',
        'Host': 'i.gasgoo.com',
        'Origin': 'https://i.gasgoo.com',
        'Pragma': 'no-cache',
        'Referer': 'https://i.gasgoo.com/data/sales/AutoModelSalesRank.aspx/GetSalesRank',
        'sec-ch-ua': '"Google Chrome";v="95", "Chromium";v="95", ";Not A Brand";v="99"',
        'sec-ch-ua-mobile': '?0',
        'sec-ch-ua-platform': '"Windows"',
        'Sec-Fetch-Dest': 'empty',
        'Sec-Fetch-Mode': 'cors',
        'Sec-Fetch-Site': 'same-origin',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/95.0.4638.69 Safari/537.36',
        'X-Requested-With': 'XMLHttpRequest'
    }
    r = requests.post(url, json=payload, headers=headers)
    data_json = r.json()
    data_json = demjson.decode(data_json['d'])
    temp_df = pd.DataFrame(data_json)
    return temp_df


if __name__ == '__main__':
    car_cpca_energy_sale_df = car_cpca_energy_sale()
    print(car_cpca_energy_sale_df)

    car_gasgoo_sale_rank_df = car_gasgoo_sale_rank(symbol="品牌榜", date="202110")
    print(car_gasgoo_sale_rank_df)

    car_gasgoo_sale_rank_df = car_gasgoo_sale_rank(symbol="车型榜", date="202104")
    print(car_gasgoo_sale_rank_df)

    car_gasgoo_sale_rank_df = car_gasgoo_sale_rank(symbol="车企榜", date="202104")
    print(car_gasgoo_sale_rank_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2020/9/28 13:53
Desc: 百度指数
感谢 https://cloudcrawler.club/categories/2019%E5%B9%B4%E6%9C%AB%E9%80%86%E5%90%91%E5%A4%8D%E4%B9%A0/
"""
import matplotlib.pyplot as plt
import pandas as pd
import requests

plt.rcParams["font.sans-serif"] = ["SimHei"]  # 显示中文标签


def decrypt(t: str, e: str) -> str:
    """
    解密函数
    :param t:
    :type t:
    :param e:
    :type e:
    :return:
    :rtype:
    """
    n, i, a, result = list(t), list(e), {}, []
    ln = int(len(n) / 2)
    start, end = n[ln:], n[:ln]
    a = dict(zip(end, start))
    return "".join([a[j] for j in e])


def get_ptbk(uniqid: str, cookie: str) -> str:
    headers = {
        "Accept": "application/json, text/plain, */*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9",
        "Cache-Control": "no-cache",
        "Cookie": cookie,
        "DNT": "1",
        "Host": "zhishu.baidu.com",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "zhishu.baidu.com",
        "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.90 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    session = requests.Session()
    session.headers.update(headers)
    with session.get(
        url=f"http://index.baidu.com/Interface/ptbk?uniqid={uniqid}"
    ) as response:
        ptbk = response.json()["data"]
        return ptbk


def baidu_search_index(word: str = "python", start_date: str = "2020-01-01", end_date: str = "2020-05-01", cookie: str = None) -> str:
    headers = {
        "Accept": "application/json, text/plain, */*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9",
        "Cache-Control": "no-cache",
        "Cookie": cookie,
        "DNT": "1",
        "Host": "zhishu.baidu.com",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "zhishu.baidu.com",
        "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.90 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    session = requests.Session()
    session.headers.update(headers)
    params = {
        "area": "0",
        "word": '[[{"name":' + f'"{word}"' + ',"wordType"' + ':1}]]',
        "startDate": start_date,
        "endDate": end_date,
    }
    with session.get(
        url="http://index.baidu.com/api/SearchApi/index", params=params
    ) as response:
        data = response.json()["data"]
        all_data = data["userIndexes"][0]["all"]["data"]
        uniqid = data["uniqid"]
        ptbk = get_ptbk(uniqid, cookie)
        result = decrypt(ptbk, all_data).split(",")
        result = [int(item) if item != "" else 0 for item in result]
        if len(result) == len(pd.date_range(start=start_date, end=end_date, freq="7D")):
            temp_df_7 = pd.DataFrame(
                [pd.date_range(start=start_date, end=end_date, freq="7D"), result],
                index=["date", word],
            ).T
            temp_df_7.index = pd.to_datetime(temp_df_7["date"])
            del temp_df_7["date"]
            return temp_df_7
        else:
            temp_df_1 = pd.DataFrame(
                [pd.date_range(start=start_date, end=end_date, freq="1D"), result],
                index=["date", word],
            ).T
            temp_df_1.index = pd.to_datetime(temp_df_1["date"])
            del temp_df_1["date"]
            return temp_df_1


def baidu_info_index(word: str, start_date: str, end_date: str, cookie: str) -> str:
    headers = {
        "Accept": "application/json, text/plain, */*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9",
        "Cache-Control": "no-cache",
        "Cookie": cookie,
        "DNT": "1",
        "Host": "zhishu.baidu.com",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "zhishu.baidu.com",
        "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.90 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    session = requests.Session()
    session.headers.update(headers)
    params = {
        "area": "0",
        "word": '[[{"name":' + f'"{word}"' + ',"wordType"' + ':1}]]',
        "startDate": start_date,
        "endDate": end_date,
    }
    with session.get(
        url=f"http://index.baidu.com/api/FeedSearchApi/getFeedIndex", params=params
    ) as response:
        data = response.json()["data"]
        all_data = data["index"][0]["data"]
        uniqid = data["uniqid"]
        ptbk = get_ptbk(uniqid, cookie)
        result = decrypt(ptbk, all_data).split(",")
        result = [int(item) if item != "" else 0 for item in result]
        if len(result) == len(pd.date_range(start=start_date, end=end_date, freq="7D")):
            temp_df_7 = pd.DataFrame(
                [pd.date_range(start=start_date, end=end_date, freq="7D"), result],
                index=["date", word],
            ).T
            temp_df_7.index = pd.to_datetime(temp_df_7["date"])
            del temp_df_7["date"]
            return temp_df_7
        else:
            temp_df_1 = pd.DataFrame(
                [pd.date_range(start=start_date, end=end_date, freq="1D"), result],
                index=["date", word],
            ).T
            temp_df_1.index = pd.to_datetime(temp_df_1["date"])
            del temp_df_1["date"]
            return temp_df_1


def baidu_media_index(word: str = "口罩", start_date: str = "2018-01-01", end_date: str = "2020-04-20", cookie: str = None) -> str:
    headers = {
        "Accept": "application/json, text/plain, */*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9",
        "Cache-Control": "no-cache",
        "Cookie": cookie,
        "DNT": "1",
        "Host": "zhishu.baidu.com",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "zhishu.baidu.com",
        "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.90 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    session = requests.Session()
    session.headers.update(headers)
    params = {
        "area": "0",
        "word": '[[{"name":' + f'"{word}"' + ',"wordType"' + ':1}]]',
        "startDate": start_date,
        "endDate": end_date,
    }
    with session.get(
        url=f"http://index.baidu.com/api/NewsApi/getNewsIndex", params=params
    ) as response:
        data = response.json()["data"]
        all_data = data["index"][0]["data"]
        uniqid = data["uniqid"]
        ptbk = get_ptbk(uniqid, cookie)
        result = decrypt(ptbk, all_data).split(",")
        result = ["0" if item == "" else item for item in result]
        result = [int(item) for item in result]
        if len(result) == len(pd.date_range(start=start_date, end=end_date, freq="7D")):
            temp_df_7 = pd.DataFrame(
                [pd.date_range(start=start_date, end=end_date, freq="7D"), result],
                index=["date", word],
            ).T
            temp_df_7.index = pd.to_datetime(temp_df_7["date"])
            del temp_df_7["date"]
            return temp_df_7
        else:
            temp_df_1 = pd.DataFrame(
                [pd.date_range(start=start_date, end=end_date, freq="1D"), result],
                index=["date", word],
            ).T
            temp_df_1.index = pd.to_datetime(temp_df_1["date"])
            del temp_df_1["date"]
            return temp_df_1


if __name__ == "__main__":
    cookie = ""
    data = baidu_search_index(
        word="python", start_date="2020-01-01", end_date="2020-09-14", cookie=cookie
    )
    print(data)
    data.dropna(inplace=True)
    data.plot()
    plt.show()
    data = baidu_info_index(
        word="口罩", start_date="2019-07-03", end_date="2020-09-21", cookie=cookie
    )
    print(data)
    data.dropna(inplace=True)
    data.plot()
    plt.show()
    data = baidu_media_index(
        word="金融科技", start_date="2020-01-01", end_date="2020-09-20", cookie=cookie
    )
    print(data)
    data.dropna(inplace=True)
    data.plot()
    plt.show()



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2021/10/30 21:12
Desc: 东方财富网-数据中心-沪深港通持股
http://data.eastmoney.com/hsgtcg/
沪深港通详情: http://finance.eastmoney.com/news/1622,20161118685370149.html
"""
import json

import pandas as pd
import requests
from bs4 import BeautifulSoup
from tqdm import tqdm

from mssdk.utils import demjson


def stock_em_hsgt_north_net_flow_in(indicator: str = "沪股通") -> pd.DataFrame:
    """
    东方财富网-数据中心-沪深港通持股-净流入
    http://data.eastmoney.com/hsgtcg/
    :param indicator: choice of {"沪股通", "深股通", "北上"}
    :type indicator: str
    :return: 东方财富网-数据中心-沪深港通持股-净流入
    :rtype: pandas.DataFrame
    """
    url = "http://push2his.eastmoney.com/api/qt/kamt.kline/get"
    params = {
        "fields1": "f1,f3,f5",
        "fields2": "f51,f52",
        "klt": "101",
        "lmt": "5000",
        "ut": "b2884a393a59ad64002292a3e90d46a5",
        "cb": "jQuery18305732402561585701_1584961751919",
        "_": "1584962164273",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = json.loads(data_text[data_text.find("{"): -2])
    if indicator == "沪股通":
        temp_df = (
            pd.DataFrame(data_json["data"]["hk2sh"])
                .iloc[:, 0]
                .str.split(",", expand=True)
        )
        temp_df.columns = ["date", "value"]
        temp_df['value'] = pd.to_numeric(temp_df['value'])
        return temp_df
    if indicator == "深股通":
        temp_df = (
            pd.DataFrame(data_json["data"]["hk2sz"])
                .iloc[:, 0]
                .str.split(",", expand=True)
        )
        temp_df.columns = ["date", "value"]
        temp_df['value'] = pd.to_numeric(temp_df['value'])
        return temp_df
    if indicator == "北上":
        temp_df = (
            pd.DataFrame(data_json["data"]["s2n"])
                .iloc[:, 0]
                .str.split(",", expand=True)
        )
        temp_df.columns = ["date", "value"]
        temp_df['value'] = pd.to_numeric(temp_df['value'])
        return temp_df


def stock_em_hsgt_north_cash(indicator: str = "沪股通") -> pd.DataFrame:
    """
    东方财富网-数据中心-沪深港通持股-资金余额
    http://data.eastmoney.com/hsgtcg/
    :param indicator: choice of {"沪股通", "深股通", "北上"}
    :type indicator: str
    :return: 东方财富网-数据中心-沪深港通持股-资金余额
    :rtype: pandas.DataFrame
    """
    url = "http://push2his.eastmoney.com/api/qt/kamt.kline/get"
    params = {
        "fields1": "f1,f3,f5",
        "fields2": "f51,f53",
        "klt": "101",
        "lmt": "5000",
        "ut": "b2884a393a59ad64002292a3e90d46a5",
        "cb": "jQuery18305732402561585701_1584961751919",
        "_": "1584962164273",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = json.loads(data_text[data_text.find("{"): -2])
    if indicator == "沪股通":
        temp_df = (
            pd.DataFrame(data_json["data"]["hk2sh"])
                .iloc[:, 0]
                .str.split(",", expand=True)
        )
        temp_df.columns = ["date", "value"]
        return temp_df
    if indicator == "深股通":
        temp_df = (
            pd.DataFrame(data_json["data"]["hk2sz"])
                .iloc[:, 0]
                .str.split(",", expand=True)
        )
        temp_df.columns = ["date", "value"]
        return temp_df
    if indicator == "北上":
        temp_df = (
            pd.DataFrame(data_json["data"]["s2n"])
                .iloc[:, 0]
                .str.split(",", expand=True)
        )
        temp_df.columns = ["date", "value"]
        return temp_df


def stock_em_hsgt_north_acc_flow_in(indicator: str = "沪股通") -> pd.DataFrame:
    """
    东方财富网-数据中心-沪深港通持股-累计净流入
    http://data.eastmoney.com/hsgtcg/
    :param indicator: choice of {"沪股通", "深股通", "北上"}
    :type indicator: str
    :return: 东方财富网-数据中心-沪深港通持股-累计净流入
    :rtype: pandas.DataFrame
    """
    url = "http://push2his.eastmoney.com/api/qt/kamt.kline/get"
    params = {
        "fields1": "f1,f3,f5",
        "fields2": "f51,f54",
        "klt": "101",
        "lmt": "5000",
        "ut": "b2884a393a59ad64002292a3e90d46a5",
        "cb": "jQuery18305732402561585701_1584961751919",
        "_": "1584962164273",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = json.loads(data_text[data_text.find("{"): -2])
    if indicator == "沪股通":
        temp_df = (
            pd.DataFrame(data_json["data"]["hk2sh"])
                .iloc[:, 0]
                .str.split(",", expand=True)
        )
        temp_df.columns = ["date", "value"]
        return temp_df
    if indicator == "深股通":
        temp_df = (
            pd.DataFrame(data_json["data"]["hk2sz"])
                .iloc[:, 0]
                .str.split(",", expand=True)
        )
        temp_df.columns = ["date", "value"]
        return temp_df
    if indicator == "北上":
        temp_df = (
            pd.DataFrame(data_json["data"]["s2n"])
                .iloc[:, 0]
                .str.split(",", expand=True)
        )
        temp_df.columns = ["date", "value"]
        return temp_df


def stock_em_hsgt_south_net_flow_in(indicator: str = "沪股通") -> pd.DataFrame:
    """
    东方财富网-数据中心-沪深港通持股-南向概括-净流入
    http://data.eastmoney.com/hsgtcg/
    :param indicator: choice of {"沪股通", "深股通", "南下"}
    :type indicator: str
    :return: 东方财富网-数据中心-沪深港通持股-南向概括-净流入
    :rtype: pandas.DataFrame
    """
    url = "http://push2his.eastmoney.com/api/qt/kamt.kline/get"
    params = {
        "fields1": "f2,f4,f6",
        "fields2": "f51,f52",
        "klt": "101",
        "lmt": "5000",
        "ut": "b2884a393a59ad64002292a3e90d46a5",
        "cb": "jQuery18307854355493858363_1584963487410",
        "_": "1584964176697",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = json.loads(data_text[data_text.find("{"): -2])
    if indicator == "沪股通":
        temp_df = (
            pd.DataFrame(data_json["data"]["sh2hk"])
                .iloc[:, 0]
                .str.split(",", expand=True)
        )
        temp_df.columns = ["date", "value"]
        return temp_df
    if indicator == "深股通":
        temp_df = (
            pd.DataFrame(data_json["data"]["sz2hk"])
                .iloc[:, 0]
                .str.split(",", expand=True)
        )
        temp_df.columns = ["date", "value"]
        return temp_df
    if indicator == "南下":
        temp_df = (
            pd.DataFrame(data_json["data"]["n2s"])
                .iloc[:, 0]
                .str.split(",", expand=True)
        )
        temp_df.columns = ["date", "value"]
        return temp_df


def stock_em_hsgt_south_cash(indicator: str = "沪股通") -> pd.DataFrame:
    """
    东方财富网-数据中心-沪深港通持股-南向概括-资金余额
    http://data.eastmoney.com/hsgtcg/
    :param indicator: choice of {"沪股通", "深股通", "南下"}
    :type indicator: str
    :return: 东方财富网-数据中心-沪深港通持股-南向概括-资金余额
    :rtype: pandas.DataFrame
    """
    url = "http://push2his.eastmoney.com/api/qt/kamt.kline/get"
    params = {
        "fields1": "f2,f4,f6",
        "fields2": "f51,f53",
        "klt": "101",
        "lmt": "5000",
        "ut": "b2884a393a59ad64002292a3e90d46a5",
        "cb": "jQuery18307854355493858363_1584963487410",
        "_": "1584964176697",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = json.loads(data_text[data_text.find("{"): -2])
    if indicator == "沪股通":
        temp_df = (
            pd.DataFrame(data_json["data"]["sh2hk"])
                .iloc[:, 0]
                .str.split(",", expand=True)
        )
        temp_df.columns = ["date", "value"]
        return temp_df
    if indicator == "深股通":
        temp_df = (
            pd.DataFrame(data_json["data"]["sz2hk"])
                .iloc[:, 0]
                .str.split(",", expand=True)
        )
        temp_df.columns = ["date", "value"]
        return temp_df
    if indicator == "南下":
        temp_df = (
            pd.DataFrame(data_json["data"]["n2s"])
                .iloc[:, 0]
                .str.split(",", expand=True)
        )
        temp_df.columns = ["date", "value"]
        return temp_df


def stock_em_hsgt_south_acc_flow_in(indicator: str = "沪股通") -> pd.DataFrame:
    """
    东方财富网-数据中心-沪深港通持股-南向概括-累计净流入
    http://data.eastmoney.com/hsgtcg/
    :param indicator: choice of {"沪股通", "深股通", "南下"}
    :type indicator: str
    :return: 东方财富网-数据中心-沪深港通持股-南向概括-累计净流入
    :rtype: pandas.DataFrame
    """
    url = "http://push2his.eastmoney.com/api/qt/kamt.kline/get"
    params = {
        "fields1": "f2,f4,f6",
        "fields2": "f51,f54",
        "klt": "101",
        "lmt": "5000",
        "ut": "b2884a393a59ad64002292a3e90d46a5",
        "cb": "jQuery18307854355493858363_1584963487410",
        "_": "1584964176697",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = json.loads(data_text[data_text.find("{"): -2])
    if indicator == "沪股通":
        temp_df = (
            pd.DataFrame(data_json["data"]["sh2hk"])
                .iloc[:, 0]
                .str.split(",", expand=True)
        )
        temp_df.columns = ["date", "value"]
        return temp_df
    if indicator == "深股通":
        temp_df = (
            pd.DataFrame(data_json["data"]["sz2hk"])
                .iloc[:, 0]
                .str.split(",", expand=True)
        )
        temp_df.columns = ["date", "value"]
        return temp_df
    if indicator == "南下":
        temp_df = (
            pd.DataFrame(data_json["data"]["n2s"])
                .iloc[:, 0]
                .str.split(",", expand=True)
        )
        temp_df.columns = ["date", "value"]
        return temp_df


def stock_em_hsgt_hold_stock(
        market: str = "北向", indicator: str = "5日排行"
) -> pd.DataFrame:
    """
    东方财富-数据中心-沪深港通持股-个股排行
    http://data.eastmoney.com/hsgtcg/list.html
    :param market: choice of {"北向", "沪股通", "深股通"}
    :type market: str
    :param indicator: choice of {"今日排行", "3日排行", "5日排行", "10日排行", "月排行", "季排行", "年排行"}
    :type indicator: str
    :return: 指定 sector 和 indicator 的数据
    :rtype: pandas.DataFrame
    """
    url = "http://data.eastmoney.com/hsgtcg/list.html"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "lxml")
    date = soup.find('div', attrs={'class': 'title'}).find("span").text.strip("（").strip("）")
    url = "http://datacenter-web.eastmoney.com/api/data/v1/get"
    if indicator == "今日排行":
        indicator_type = "1"
    if indicator == "3日排行":
        indicator_type = "3"
    if indicator == "5日排行":
        indicator_type = "5"
    if indicator == "10日排行":
        indicator_type = "10"
    if indicator == "月排行":
        indicator_type = "M"
    if indicator == "季排行":
        indicator_type = "Q"
    if indicator == "年排行":
        indicator_type = "Y"
    if market == "北向":
        filter_str = f"""(TRADE_DATE='{date}')(INTERVAL_TYPE="{indicator_type}")"""
    elif market == "沪股通":
        filter_str = f"""(TRADE_DATE='{date}')(INTERVAL_TYPE="{indicator_type}")(MUTUAL_TYPE="001")"""
    elif market == "深股通":
        filter_str = f"""(TRADE_DATE='{date}')(INTERVAL_TYPE="{indicator_type}")(MUTUAL_TYPE="001")"""
    params = {
        'sortColumns': 'ADD_MARKET_CAP',
        'sortTypes': '-1',
        'pageSize': '50000',
        'pageNumber': '1',
        'reportName': 'RPT_MUTUAL_STOCK_NORTHSTA',
        'columns': 'ALL',
        'source': 'WEB',
        'client': 'WEB',
        'filter': filter_str
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    page_num = data_json['result']['pages']
    big_df = pd.DataFrame()
    for page in tqdm(range(1, page_num + 1)):
        params.update({'pageNumber': page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json['result']["data"])
        big_df = big_df.append(temp_df, ignore_index=True)

    big_df.reset_index(inplace=True)
    big_df['index'] = range(1, len(big_df) + 1)
    big_df.columns = [
        '序号',
        '_',
        '_',
        '日期',
        '_',
        '名称',
        '_',
        '_',
        '代码',
        '_',
        '_',
        '_',
        '_',
        '今日持股-股数',
        '今日持股-市值',
        '今日持股-占流通股比',
        '今日持股-占总股本比',
        '今日收盘价',
        '今日涨跌幅',
        '_',
        '所属板块',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
        f'{indicator.split("排")[0]}增持估计-市值',
        f'{indicator.split("排")[0]}增持估计-股数',
        f'{indicator.split("排")[0]}增持估计-市值增幅',
        f'{indicator.split("排")[0]}增持估计-占流通股比',
        f'{indicator.split("排")[0]}增持估计-占总股本比',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
    ]
    big_df = big_df[[
        '序号',
        '代码',
        '名称',
        '今日收盘价',
        '今日涨跌幅',
        '今日持股-股数',
        '今日持股-市值',
        '今日持股-占流通股比',
        '今日持股-占总股本比',
        f'{indicator.split("排")[0]}增持估计-股数',
        f'{indicator.split("排")[0]}增持估计-市值',
        f'{indicator.split("排")[0]}增持估计-市值增幅',
        f'{indicator.split("排")[0]}增持估计-占流通股比',
        f'{indicator.split("排")[0]}增持估计-占总股本比',
        '所属板块',
        '日期',
    ]]
    big_df['今日收盘价'] = pd.to_numeric(big_df['今日收盘价'])
    big_df['今日涨跌幅'] = pd.to_numeric(big_df['今日涨跌幅'])
    big_df['今日持股-股数'] = pd.to_numeric(big_df['今日持股-股数'])
    big_df['今日持股-市值'] = pd.to_numeric(big_df['今日持股-市值'])
    big_df['今日持股-占流通股比'] = pd.to_numeric(big_df['今日持股-占流通股比'])
    big_df['今日持股-占总股本比'] = pd.to_numeric(big_df['今日持股-占总股本比'])
    big_df[f'{indicator.split("排")[0]}增持估计-股数'] = pd.to_numeric(big_df[f'{indicator.split("排")[0]}增持估计-股数'])
    big_df[f'{indicator.split("排")[0]}增持估计-市值'] = pd.to_numeric(big_df[f'{indicator.split("排")[0]}增持估计-市值'])
    big_df[f'{indicator.split("排")[0]}增持估计-市值增幅'] = pd.to_numeric(big_df[f'{indicator.split("排")[0]}增持估计-市值增幅'])
    big_df[f'{indicator.split("排")[0]}增持估计-占流通股比'] = pd.to_numeric(big_df[f'{indicator.split("排")[0]}增持估计-占流通股比'])
    big_df[f'{indicator.split("排")[0]}增持估计-占总股本比'] = pd.to_numeric(big_df[f'{indicator.split("排")[0]}增持估计-占总股本比'])
    big_df['日期'] = pd.to_datetime(big_df['日期']).dt.date
    return big_df


def stock_em_hsgt_stock_statistics(
        symbol: str = "南向持股", start_date: str = "20211027", end_date: str = "20211027"
):
    """
    东方财富网-数据中心-沪深港通-沪深港通持股-每日个股统计
    http://data.eastmoney.com/hsgtcg/StockStatistics.aspx
    market=001, 沪股通持股
    market=003, 深股通持股
    :param symbol: choice of {"北向持股", "南向持股"}
    :type symbol: str
    :param start_date: 指定数据获取开始的时间, e.g., "20200713"
    :type start_date: str
    :param end_date: 指定数据获取结束的时间, e.g., "20200715"
    :type end_date:str
    :return: 指定市场和指定时间段的每日个股统计数据
    :rtype: pandas.DataFrame
    """
    start_date = "-".join([start_date[:4], start_date[4:6], start_date[6:]])
    end_date = "-".join([end_date[:4], end_date[4:6], end_date[6:]])
    if symbol == "南向持股":
        params = {
            'sortColumns': 'TRADE_DATE',
            'sortTypes': '-1',
            'pageSize': '1000',
            'pageNumber': '1',
            'columns': 'ALL',
            'source': 'WEB',
            'client': 'WEB',
            "filter": f"""(INTERVAL_TYPE="1")(RN=1)(TRADE_DATE>='{start_date}')(TRADE_DATE<='{end_date}')""",
            "rt": "53160469",
            'reportName': 'RPT_MUTUAL_STOCK_HOLDRANKS',
        }
        if start_date == end_date:
            params.update({"filter": f"""(INTERVAL_TYPE="1")(RN=1)(TRADE_DATE='{start_date}')"""})
        url = "http://datacenter-web.eastmoney.com/api/data/v1/get"
        r = requests.get(url, params=params)
        data_json = r.json()
        total_page = data_json['result']['pages']
        big_df = pd.DataFrame()
        for page in tqdm(range(1, int(total_page)+1), leave=False):
            params.update({'pageNumber': page})
            r = requests.get(url, params=params)
            data_json = r.json()
            temp_df = pd.DataFrame(data_json["result"]['data'])
            big_df = big_df.append(temp_df, ignore_index=True)
        big_df.columns = [
            '-',
            '持股日期',
            '-',
            '-',
            '股票简称',
            '股票代码',
            '-',
            '-',
            '-',
            '-',
            '持股数量',
            '持股市值',
            '-',
            '-',
            '-',
            '-',
            '当日收盘价',
            '当日涨跌幅',
            '-',
            '-',
            '-',
            '-',
            '-',
            '持股数量占发行股百分比',
            '-',
            '持股市值变化-1日',
            '持股市值变化-5日',
            '持股市值变化-10日',
            '-',
        ]
        big_df = big_df[[
            '持股日期',
            '股票代码',
            '股票简称',
            '当日收盘价',
            '当日涨跌幅',
            '持股数量',
            '持股市值',
            '持股数量占发行股百分比',
            '持股市值变化-1日',
            '持股市值变化-5日',
            '持股市值变化-10日',
        ]]
        big_df['持股日期'] = pd.to_datetime(big_df['持股日期']).dt.date
        big_df['当日收盘价'] = pd.to_numeric(big_df['当日收盘价'])
        big_df['当日涨跌幅'] = pd.to_numeric(big_df['当日涨跌幅'])
        big_df['持股数量'] = pd.to_numeric(big_df['持股数量'])
        big_df['持股市值'] = pd.to_numeric(big_df['持股市值'])
        big_df['持股数量占发行股百分比'] = pd.to_numeric(big_df['持股数量占发行股百分比'])
        big_df['持股市值变化-1日'] = pd.to_numeric(big_df['持股市值变化-1日'])
        big_df['持股市值变化-5日'] = pd.to_numeric(big_df['持股市值变化-5日'])
        big_df['持股市值变化-10日'] = pd.to_numeric(big_df['持股市值变化-10日'])
    elif symbol == "北向持股":
        params = {
            'sortColumns': 'TRADE_DATE',
            'sortTypes': '-1',
            'pageSize': '1000',
            'pageNumber': '1',
            'columns': 'ALL',
            'source': 'WEB',
            'client': 'WEB',
            "filter": f"""(INTERVAL_TYPE="1")(MUTUAL_TYPE in ("001","003"))(TRADE_DATE>='{start_date}')(TRADE_DATE<='{end_date}')""",
            "rt": "53160469",
            'reportName': 'RPT_MUTUAL_STOCK_NORTHSTA',
        }
        if start_date == end_date:
            params.update({"filter": f"""(INTERVAL_TYPE="1")(MUTUAL_TYPE in ("001","003"))(TRADE_DATE='{start_date}')"""})
        url = "http://datacenter-web.eastmoney.com/api/data/v1/get"
        r = requests.get(url, params=params)
        data_json = r.json()
        total_page = data_json['result']['pages']
        big_df = pd.DataFrame()
        for page in tqdm(range(1, int(total_page)+1), leave=False):
            params.update({'pageNumber': page})
            r = requests.get(url, params=params)
            data_json = r.json()
            temp_df = pd.DataFrame(data_json["result"]['data'])
            big_df = big_df.append(temp_df, ignore_index=True)
        big_df.columns = [
            '-',
            '-',
            '持股日期',
            '-',
            '股票简称',
            '-',
            '-',
            '股票代码',
            '-',
            '-',
            '-',
            '-',
            '持股数量',
            '持股市值',
            '-',
            '持股数量占发行股百分比',
            '当日收盘价',
            '当日涨跌幅',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '持股市值变化-1日',
            '持股市值变化-5日',
            '持股市值变化-10日',
            '-',
            '-',
        ]
        big_df = big_df[[
            '持股日期',
            '股票代码',
            '股票简称',
            '当日收盘价',
            '当日涨跌幅',
            '持股数量',
            '持股市值',
            '持股数量占发行股百分比',
            '持股市值变化-1日',
            '持股市值变化-5日',
            '持股市值变化-10日',
        ]]
        big_df['持股日期'] = pd.to_datetime(big_df['持股日期']).dt.date
        big_df['当日收盘价'] = pd.to_numeric(big_df['当日收盘价'])
        big_df['当日涨跌幅'] = pd.to_numeric(big_df['当日涨跌幅'])
        big_df['持股数量'] = pd.to_numeric(big_df['持股数量'])
        big_df['持股市值'] = pd.to_numeric(big_df['持股市值'])
        big_df['持股数量占发行股百分比'] = pd.to_numeric(big_df['持股数量占发行股百分比'])
        big_df['持股市值变化-1日'] = pd.to_numeric(big_df['持股市值变化-1日'])
        big_df['持股市值变化-5日'] = pd.to_numeric(big_df['持股市值变化-5日'])
        big_df['持股市值变化-10日'] = pd.to_numeric(big_df['持股市值变化-10日'])
    elif symbol == "沪股通持股":
        params = {
            'sortColumns': 'TRADE_DATE',
            'sortTypes': '-1',
            'pageSize': '1000',
            'pageNumber': '1',
            'columns': 'ALL',
            'source': 'WEB',
            'client': 'WEB',
            "filter": f"""(INTERVAL_TYPE="1")(MUTUAL_TYPE="001")(TRADE_DATE>='{start_date}')(TRADE_DATE<='{end_date}')""",
            'reportName': 'RPT_MUTUAL_STOCK_NORTHSTA',
        }
        if start_date == end_date:
            params.update({"filter": f"""(INTERVAL_TYPE="1")(MUTUAL_TYPE="001")(TRADE_DATE='{start_date}')"""})
        url = "http://datacenter-web.eastmoney.com/api/data/v1/get"
        r = requests.get(url, params=params)
        data_json = r.json()
        total_page = data_json['result']['pages']
        big_df = pd.DataFrame()
        for page in tqdm(range(1, int(total_page)+1), leave=False):
            params.update({'pageNumber': page})
            r = requests.get(url, params=params)
            data_json = r.json()
            temp_df = pd.DataFrame(data_json["result"]['data'])
            big_df = big_df.append(temp_df, ignore_index=True)
        big_df.columns = [
            '-',
            '-',
            '持股日期',
            '-',
            '股票简称',
            '-',
            '-',
            '股票代码',
            '-',
            '-',
            '-',
            '-',
            '持股数量',
            '持股市值',
            '-',
            '持股数量占发行股百分比',
            '当日收盘价',
            '当日涨跌幅',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '持股市值变化-1日',
            '持股市值变化-5日',
            '持股市值变化-10日',
            '-',
            '-',
        ]
        big_df = big_df[[
            '持股日期',
            '股票代码',
            '股票简称',
            '当日收盘价',
            '当日涨跌幅',
            '持股数量',
            '持股市值',
            '持股数量占发行股百分比',
            '持股市值变化-1日',
            '持股市值变化-5日',
            '持股市值变化-10日',
        ]]
        big_df['持股日期'] = pd.to_datetime(big_df['持股日期']).dt.date
        big_df['当日收盘价'] = pd.to_numeric(big_df['当日收盘价'])
        big_df['当日涨跌幅'] = pd.to_numeric(big_df['当日涨跌幅'])
        big_df['持股数量'] = pd.to_numeric(big_df['持股数量'])
        big_df['持股市值'] = pd.to_numeric(big_df['持股市值'])
        big_df['持股数量占发行股百分比'] = pd.to_numeric(big_df['持股数量占发行股百分比'])
        big_df['持股市值变化-1日'] = pd.to_numeric(big_df['持股市值变化-1日'])
        big_df['持股市值变化-5日'] = pd.to_numeric(big_df['持股市值变化-5日'])
        big_df['持股市值变化-10日'] = pd.to_numeric(big_df['持股市值变化-10日'])
    elif symbol == "深股通持股":
        params = {
            'sortColumns': 'TRADE_DATE',
            'sortTypes': '-1',
            'pageSize': '1000',
            'pageNumber': '1',
            'columns': 'ALL',
            'source': 'WEB',
            'client': 'WEB',
            "filter": f"""(INTERVAL_TYPE="1")(MUTUAL_TYPE="003")(TRADE_DATE>='{start_date}')(TRADE_DATE<='{end_date}')""",
            'reportName': 'RPT_MUTUAL_STOCK_NORTHSTA',
        }
        if start_date == end_date:
            params.update({"filter": f"""(INTERVAL_TYPE="1")(MUTUAL_TYPE="003")(TRADE_DATE='{start_date}')"""})
        url = "http://datacenter-web.eastmoney.com/api/data/v1/get"
        r = requests.get(url, params=params)
        data_json = r.json()
        total_page = data_json['result']['pages']
        big_df = pd.DataFrame()
        for page in tqdm(range(1, int(total_page)+1), leave=False):
            params.update({'pageNumber': page})
            r = requests.get(url, params=params)
            data_json = r.json()
            temp_df = pd.DataFrame(data_json["result"]['data'])
            big_df = big_df.append(temp_df, ignore_index=True)
        big_df.columns = [
            '-',
            '-',
            '持股日期',
            '-',
            '股票简称',
            '-',
            '-',
            '股票代码',
            '-',
            '-',
            '-',
            '-',
            '持股数量',
            '持股市值',
            '-',
            '持股数量占发行股百分比',
            '当日收盘价',
            '当日涨跌幅',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '-',
            '持股市值变化-1日',
            '持股市值变化-5日',
            '持股市值变化-10日',
            '-',
            '-',
        ]
        big_df = big_df[[
            '持股日期',
            '股票代码',
            '股票简称',
            '当日收盘价',
            '当日涨跌幅',
            '持股数量',
            '持股市值',
            '持股数量占发行股百分比',
            '持股市值变化-1日',
            '持股市值变化-5日',
            '持股市值变化-10日',
        ]]
        big_df['持股日期'] = pd.to_datetime(big_df['持股日期']).dt.date
        big_df['当日收盘价'] = pd.to_numeric(big_df['当日收盘价'])
        big_df['当日涨跌幅'] = pd.to_numeric(big_df['当日涨跌幅'])
        big_df['持股数量'] = pd.to_numeric(big_df['持股数量'])
        big_df['持股市值'] = pd.to_numeric(big_df['持股市值'])
        big_df['持股数量占发行股百分比'] = pd.to_numeric(big_df['持股数量占发行股百分比'])
        big_df['持股市值变化-1日'] = pd.to_numeric(big_df['持股市值变化-1日'])
        big_df['持股市值变化-5日'] = pd.to_numeric(big_df['持股市值变化-5日'])
        big_df['持股市值变化-10日'] = pd.to_numeric(big_df['持股市值变化-10日'])
    return big_df


def stock_em_hsgt_institution_statistics(
        market: str = "北向持股", start_date: str = "20200127", end_date: str = "20200127"
):
    """
    东方财富网-数据中心-沪深港通-沪深港通持股-每日机构统计
    http://data.eastmoney.com/hsgtcg/InstitutionStatistics.aspx
    :param market: choice of {"北向持股", "南向持股", "沪股通持股", "深股通持股"}
    :type market: str
    :param start_date: 指定数据获取开始的时间, e.g., "20200713"
    :type start_date: str
    :param end_date: 指定数据获取结束的时间, e.g., "20200715"
    :type end_date:str
    :return: 指定市场和指定时间段的每日个股统计数据
    :rtype: pandas.DataFrame
    """
    start_date = "-".join([start_date[:4], start_date[4:6], start_date[6:]])
    end_date = "-".join([end_date[:4], end_date[4:6], end_date[6:]])
    if market == "南向持股":
        params = {
            "type": "HSGTCOMSTA",
            "token": "70f12f2f4f091e459a279469fe49eca5",
            "st": "HDDATE,SHAREHOLDCOUNT",
            "sr": "3",
            "p": "1",
            "ps": "5000",
            "js": "var gvfJjbLz={pages:(tp),data:(x)}",
            "filter": f"(MARKET='S')(HDDATE>=^{start_date}^ and HDDATE<=^{end_date}^)",
            "rt": "53160469",
        }
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.116 Safari/537.36"
        }
        url = "http://dcfm.eastmoney.com//em_mutisvcexpandinterface/api/js/get"
        r = requests.get(url, params=params, headers=headers)
        data_text = r.text
        data_json = demjson.decode(data_text[data_text.find("{"):])
        temp_df = pd.DataFrame(data_json["data"])
        temp_df.columns = [
            '持股日期',
            '_',
            '机构名称',
            '持股只数',
            '持股市值',
            '持股市值变化-1日',
            '持股市值变化-5日',
            '持股市值变化-10日',
            '_',
        ]
        temp_df = temp_df[[
            '持股日期',
            '机构名称',
            '持股只数',
            '持股市值',
            '持股市值变化-1日',
            '持股市值变化-5日',
            '持股市值变化-10日',
        ]]
        temp_df['持股日期'] = temp_df['持股日期'].str.slice(0, 10)
        return temp_df

    elif market == "北向持股":
        params = {
            "type": "HSGTCOMSTA",
            "token": "70f12f2f4f091e459a279469fe49eca5",
            "st": "HDDATE,SHAREHOLDCOUNT",
            "sr": "3",
            "p": "1",
            "ps": "5000",
            "js": "var gvfJjbLz={pages:(tp),data:(x)}",
            "filter": f"(MARKET='N')(HDDATE>=^{start_date}^ and HDDATE<=^{end_date}^)",
            "rt": "53160469",
        }
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.116 Safari/537.36"
        }
        url = "http://dcfm.eastmoney.com//em_mutisvcexpandinterface/api/js/get"
        r = requests.get(url, params=params, headers=headers)
        data_text = r.text
        data_json = demjson.decode(data_text[data_text.find("{"):])
        temp_df = pd.DataFrame(data_json["data"])
        temp_df.columns = [
            '持股日期',
            '_',
            '机构名称',
            '持股只数',
            '持股市值',
            '持股市值变化-1日',
            '持股市值变化-5日',
            '持股市值变化-10日',
            '_',
        ]
        temp_df = temp_df[[
            '持股日期',
            '机构名称',
            '持股只数',
            '持股市值',
            '持股市值变化-1日',
            '持股市值变化-5日',
            '持股市值变化-10日',
        ]]
        temp_df['持股日期'] = temp_df['持股日期'].str.slice(0, 10)
        return temp_df
    elif market == "沪股通持股":
        params = {
            "type": "HSGTCOMSTA",
            "token": "70f12f2f4f091e459a279469fe49eca5",
            "st": "HDDATE,SHAREHOLDCOUNT",
            "sr": "3",
            "p": "1",
            "ps": "5000",
            "js": "var gvfJjbLz={pages:(tp),data:(x)}",
            "filter": f"(MARKET='001')(HDDATE>=^{start_date}^ and HDDATE<=^{end_date}^)",
            "rt": "53160469",
        }
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.116 Safari/537.36"
        }
        url = "http://dcfm.eastmoney.com//em_mutisvcexpandinterface/api/js/get"
        r = requests.get(url, params=params, headers=headers)
        data_text = r.text
        data_json = demjson.decode(data_text[data_text.find("{"):])
        temp_df = pd.DataFrame(data_json["data"])
        temp_df.columns = [
            '持股日期',
            '_',
            '机构名称',
            '持股只数',
            '持股市值',
            '持股市值变化-1日',
            '持股市值变化-5日',
            '持股市值变化-10日',
            '_',
        ]
        temp_df = temp_df[[
            '持股日期',
            '机构名称',
            '持股只数',
            '持股市值',
            '持股市值变化-1日',
            '持股市值变化-5日',
            '持股市值变化-10日',
        ]]
        temp_df['持股日期'] = temp_df['持股日期'].str.slice(0, 10)
        return temp_df
    elif market == "深股通持股":
        params = {
            "type": "HSGTCOMSTA",
            "token": "70f12f2f4f091e459a279469fe49eca5",
            "st": "HDDATE,SHAREHOLDCOUNT",
            "sr": "3",
            "p": "1",
            "ps": "5000",
            "js": "var gvfJjbLz={pages:(tp),data:(x)}",
            "filter": f"(MARKET='003')(HDDATE>=^{start_date}^ and HDDATE<=^{end_date}^)",
            "rt": "53160469",
        }
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.116 Safari/537.36"
        }
        url = "http://dcfm.eastmoney.com//em_mutisvcexpandinterface/api/js/get"
        r = requests.get(url, params=params, headers=headers)
        data_text = r.text
        data_json = demjson.decode(data_text[data_text.find("{"):])
        temp_df = pd.DataFrame(data_json["data"])
        temp_df.columns = [
            '持股日期',
            '_',
            '机构名称',
            '持股只数',
            '持股市值',
            '持股市值变化-1日',
            '持股市值变化-5日',
            '持股市值变化-10日',
            '_',
        ]
        temp_df = temp_df[[
            '持股日期',
            '机构名称',
            '持股只数',
            '持股市值',
            '持股市值变化-1日',
            '持股市值变化-5日',
            '持股市值变化-10日',
        ]]
        temp_df['持股日期'] = temp_df['持股日期'].str.slice(0, 10)
        return temp_df


def stock_em_hsgt_hist(symbol: str = "沪股通") -> pd.DataFrame:
    """
    东方财富网-数据中心-资金流向-沪深港通资金流向-沪深港通历史数据
    http://data.eastmoney.com/hsgt/index.html
    :param symbol: choice of {"沪股通", "深股通", "港股通沪", "港股通深"}
    :type symbol: str
    :return: 沪深港通历史数据
    :rtype: pandas.DataFrame
    """
    symbol_map = {"沪股通": "1", "深股通": "3", "港股通沪": "2", "港股通深": "4"}
    url = "http://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        'sortColumns': 'TRADE_DATE',
        'sortTypes': '-1',
        'pageSize': '1000',
        'pageNumber': '1',
        'reportName': 'RPT_MUTUAL_DEAL_HISTORY',
        'columns': 'ALL',
        'source': 'WEB',
        'client': 'WEB',
        'filter': f'(MUTUAL_TYPE="00{symbol_map[symbol]}")',
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json['result']['pages']
    big_df = pd.DataFrame()
    for page in tqdm(range(1, int(total_page)+1), leave=False):
        params.update({'pageNumber': page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]['data'])
        big_df = big_df.append(temp_df, ignore_index=True)
    if symbol == "沪股通":
        index_name = "上证指数"
    elif symbol == "深股通":
        index_name = "深证指数"
    else:
        index_name = "恒生指数"

    big_df.rename(columns={
        'MUTUAL_TYPE': '-',
        'TRADE_DATE': '日期',
        'FUND_INFLOW': '当日资金流入',
        'NET_DEAL_AMT': '当日成交净买额',
        'QUOTA_BALANCE': '当日余额',
        'ACCUM_DEAL_AMT': '历史累计净买额',
        'BUY_AMT': '买入成交额',
        'SELL_AMT': '卖出成交额',
        'LEAD_STOCKS_CODE': '-',
        'LEAD_STOCKS_NAME': '领涨股',
        'LS_CHANGE_RATE': '领涨股涨跌幅',
        'INDEX_CLOSE_PRICE': index_name,
        'INDEX_CHANGE_RATE': '涨跌幅'
    }, inplace=True)
    big_df = big_df[[
        "日期",
        "当日成交净买额",
        "买入成交额",
        "卖出成交额",
        "历史累计净买额",
        "当日资金流入",
        "当日余额",
        "领涨股",
        "领涨股涨跌幅",
        index_name,
        "涨跌幅",
    ]]
    big_df['日期'] = pd.to_datetime(big_df['日期']).dt.date
    big_df['当日资金流入'] = pd.to_numeric(big_df['当日资金流入']) / 100
    big_df['当日余额'] = pd.to_numeric(big_df['当日余额']) / 100
    if symbol == "沪股通" or symbol == "深股通":
        big_df['历史累计净买额'] = pd.to_numeric(big_df['历史累计净买额']) / 100
    else:
        big_df['历史累计净买额'] = pd.to_numeric(big_df['历史累计净买额']) / 100 / 10000
    big_df['当日成交净买额'] = pd.to_numeric(big_df['当日成交净买额']) / 100
    big_df['买入成交额'] = pd.to_numeric(big_df['买入成交额']) / 100
    big_df['卖出成交额'] = pd.to_numeric(big_df['卖出成交额']) / 100
    big_df['领涨股涨跌幅'] = pd.to_numeric(big_df['领涨股涨跌幅'])
    big_df[index_name] = pd.to_numeric(big_df[index_name])
    big_df['涨跌幅'] = pd.to_numeric(big_df['涨跌幅'])
    return big_df


def stock_em_hsgt_board_rank(symbol: str = "北向资金增持行业板块排行", indicator: str = "今日") -> pd.DataFrame:
    """
    东方财富网-数据中心-沪深港通持股-行业板块排行-北向资金增持行业板块排行
    http://data.eastmoney.com/hsgtcg/hy.html
    :param symbol: choice of {"北向资金增持行业板块排行", "北向资金增持概念板块排行", "北向资金增持地域板块排行"}
    :type symbol: str
    :param indicator: choice of {"今日", "3日", "5日", "10日", "1月", "1季", "1年"}
    :type indicator: str
    :return: 北向资金增持行业板块排行
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "北向资金增持行业板块排行": "HSGT20_HYTJ_SUM",
        "北向资金增持概念板块排行": "HSGT20_GNTJ_SUM",
        "北向资金增持地域板块排行": "HSGT20_DQTJ_SUM",
    }
    indicator_map = {
        "今日": "1",
        "3日": "3",
        "5日": "5",
        "10日": "10",
        "1月": "m",
        "1季": "jd",
        "1年": "y",
    }
    url = "http://dcfm.eastmoney.com/EM_MutiSvcExpandInterface/api/js/get"
    params = {
        "type": symbol_map[symbol],
        "token": "894050c76af8597a853f5b408b759f5d",
        "st": "ShareSZ_ZC",
        "sr": "-1",
        "p": "1",
        "ps": "5000",
        "js": "var WCCFPIdQ={pages:(tp),data:(x)}",
        "filter": f"(DateType='{indicator_map[indicator]}')",
        "rt": "53477178",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = demjson.decode(data_text[data_text.find("{"):])
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = list(range(1, len(temp_df) + 1))
    temp_df.columns = [
        "序号",
        "_",
        "_",
        "名称",
        "_",
        "最新涨跌幅",
        "北向资金今日增持估计-股票只数",
        "北向资金今日持股-股票只数",
        "北向资金今日持股-占北向资金比",
        "北向资金今日增持估计-市值",
        "北向资金今日增持估计-市值增幅",
        "北向资金今日增持估计-占板块比",
        "北向资金今日增持估计-占北向资金比",
        "_",
        "北向资金今日持股-市值",
        "_",
        "今日增持最大股-市值",
        "_",
        "_",
        "_",
        "今日减持最大股-市值",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "北向资金今日持股-占板块比",
        "今日增持最大股-占股本比",
        "_",
        "今日减持最大股-占股本比",
        "_",
    ]
    temp_df = temp_df[
        [
            "序号",
            "名称",
            "最新涨跌幅",
            "北向资金今日持股-股票只数",
            "北向资金今日持股-市值",
            "北向资金今日持股-占板块比",
            "北向资金今日持股-占北向资金比",
            "北向资金今日增持估计-股票只数",
            "北向资金今日增持估计-市值",
            "北向资金今日增持估计-市值增幅",
            "北向资金今日增持估计-占板块比",
            "北向资金今日增持估计-占北向资金比",
            "今日增持最大股-市值",
            "今日增持最大股-占股本比",
            "今日减持最大股-占股本比",
            "今日减持最大股-市值",
        ]
    ]
    return temp_df


def stock_hsgt_individual_em(stock: str = "002008") -> pd.DataFrame:
    """
    东方财富-数据中心-沪深港通-沪深港通持股-具体股票
    http://data.eastmoney.com/hsgtcg/StockHdStatistics/002008.html
    :param stock: 股票代码
    :type stock: str
    :return: 具体股票-沪深港通持股
    :rtype: pandas.DataFrame
    """
    url = "http://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "TRADE_DATE",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_MUTUAL_HOLDSTOCKNORTH_STA",
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "filter": f"""(SECURITY_CODE="{stock}")(TRADE_DATE>='2020-07-25')""",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json['result']['data'])
    temp_df.rename(columns={
        'SECURITY_INNER_CODE': '-',
        'SECUCODE': '-',
        'TRADE_DATE': '持股日期',
        'SECURITY_CODE': '-',
        'SECURITY_NAME': '-',
        'MUTUAL_TYPE': '-',
        'CHANGE_RATE': '当日涨跌幅',
        'CLOSE_PRICE': '当日收盘价',
        'HOLD_SHARES': '持股数量',
        'HOLD_MARKET_CAP': '持股市值',
        'A_SHARES_RATIO': '-',
        'HOLD_SHARES_RATIO': '持股数量占A股百分比',
        'FREE_SHARES_RATIO': '-',
        'TOTAL_SHARES_RATIO': '-',
        'HOLD_MARKETCAP_CHG1': '持股市值变化-1日',
        'HOLD_MARKETCAP_CHG5': '持股市值变化-5日',
        'HOLD_MARKETCAP_CHG10': '持股市值变化-10日',
    }, inplace=True)
    temp_df = temp_df[[
        '持股日期',
        '当日收盘价',
        '当日涨跌幅',
        '持股数量',
        '持股市值',
        '持股数量占A股百分比',
        '持股市值变化-1日',
        '持股市值变化-5日',
        '持股市值变化-10日',
    ]]
    temp_df['持股日期'] = pd.to_datetime(temp_df['持股日期']).dt.date
    temp_df['当日收盘价'] = pd.to_numeric(temp_df['当日收盘价'])
    temp_df['当日涨跌幅'] = pd.to_numeric(temp_df['当日涨跌幅'])
    temp_df['持股数量'] = pd.to_numeric(temp_df['持股数量'])
    temp_df['持股市值'] = pd.to_numeric(temp_df['持股市值'])
    temp_df['持股数量占A股百分比'] = pd.to_numeric(temp_df['持股数量占A股百分比'])
    temp_df['持股市值变化-1日'] = pd.to_numeric(temp_df['持股市值变化-1日'])
    temp_df['持股市值变化-5日'] = pd.to_numeric(temp_df['持股市值变化-5日'])
    temp_df['持股市值变化-10日'] = pd.to_numeric(temp_df['持股市值变化-10日'])
    return temp_df


def stock_hsgt_individual_detail_em(stock: str = "600596", start_date: str = "20210830", end_date: str = "20211026") -> pd.DataFrame:
    """
    东方财富-数据中心-沪深港通-沪深港通持股-具体股票详情
    http://data.eastmoney.com/hsgtcg/StockHdStatistics/002008.html
    :param stock: 股票代码
    :type stock: str
    :param start_date: 开始时间
    :type start_date: str
    :param end_date: 结束时间
    :type end_date: str
    :return: 沪深港通持股-具体股票详情
    :rtype: pandas.DataFrame
    """
    url = "http://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "HOLD_DATE",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_MUTUAL_HOLD_DET",
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "filter": f"""(SECURITY_CODE="{stock}")(MARKET_CODE="003")(HOLD_DATE>='{'-'.join([start_date[:4], start_date[4:6], start_date[6:]])}')(HOLD_DATE<='{'-'.join([end_date[:4], end_date[4:6], end_date[6:]])}')""",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    try:
        data_json['result']['pages']
    except TypeError as e:
        params.update({
            "filter": f"""(SECURITY_CODE="{stock}")(MARKET_CODE="001")(HOLD_DATE>='{'-'.join([start_date[:4], start_date[4:6], start_date[6:]])}')(HOLD_DATE<='{'-'.join([end_date[:4], end_date[4:6], end_date[6:]])}')""",
        })
        r = requests.get(url, params=params)
        data_json = r.json()
    total_page = data_json['result']['pages']
    big_df = pd.DataFrame()
    for page in tqdm(range(1, int(total_page)+1), leave=False):
        params.update({'pageNumber': page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json['result']['data'])
        big_df = big_df.append(temp_df, ignore_index=True)
    big_df.rename(columns={
        'SECUCODE': '-',
        'SECURITY_CODE': '-',
        'SECURITY_INNER_CODE': '-',
        'SECURITY_NAME_ABBR': '-',
        'HOLD_DATE': '持股日期',
        'ORG_CODE': '-',
        'ORG_NAME': '机构名称',
        'HOLD_NUM': '持股数量',
        'MARKET_CODE': '-',
        'HOLD_SHARES_RATIO': '持股数量占A股百分比',
        'HOLD_MARKET_CAP': '持股市值',
        'CLOSE_PRICE': '当日收盘价',
        'CHANGE_RATE': '当日涨跌幅',
        'HOLD_MARKET_CAPONE': '持股市值变化-1日',
        'HOLD_MARKET_CAPFIVE': '持股市值变化-5日',
        'HOLD_MARKET_CAPTEN': '持股市值变化-10日',
        'PARTICIPANT_CODE': '-',
    }, inplace=True)
    big_df = big_df[[
        '持股日期',
        '当日收盘价',
        '当日涨跌幅',
        '机构名称',
        '持股数量',
        '持股市值',
        '持股数量占A股百分比',
        '持股市值变化-1日',
        '持股市值变化-5日',
        '持股市值变化-10日',
    ]]
    big_df['持股日期'] = pd.to_datetime(big_df['持股日期']).dt.date
    big_df['当日收盘价'] = pd.to_numeric(big_df['当日收盘价'])
    big_df['当日涨跌幅'] = pd.to_numeric(big_df['当日涨跌幅'])
    big_df['持股数量'] = pd.to_numeric(big_df['持股数量'])
    big_df['持股市值'] = pd.to_numeric(big_df['持股市值'])
    big_df['持股数量占A股百分比'] = pd.to_numeric(big_df['持股数量占A股百分比'])
    big_df['持股市值变化-1日'] = pd.to_numeric(big_df['持股市值变化-1日'])
    big_df['持股市值变化-5日'] = pd.to_numeric(big_df['持股市值变化-5日'])
    big_df['持股市值变化-10日'] = pd.to_numeric(big_df['持股市值变化-10日'])
    return big_df


if __name__ == "__main__":
    stock_em_hsgt_north_net_flow_in_df = stock_em_hsgt_north_net_flow_in(indicator="沪股通")
    print(stock_em_hsgt_north_net_flow_in_df)

    stock_em_hsgt_north_net_flow_in_df = stock_em_hsgt_north_net_flow_in(indicator="深股通")
    print(stock_em_hsgt_north_net_flow_in_df)

    stock_em_hsgt_north_net_flow_in_df = stock_em_hsgt_north_net_flow_in(indicator="北上")
    print(stock_em_hsgt_north_net_flow_in_df)

    stock_em_hsgt_north_cash_df = stock_em_hsgt_north_cash(indicator="沪股通")
    print(stock_em_hsgt_north_cash_df)

    stock_em_hsgt_north_acc_flow_in_df = stock_em_hsgt_north_acc_flow_in(
        indicator="北上"
    )
    print(stock_em_hsgt_north_acc_flow_in_df)

    stock_em_hsgt_south_net_flow_in_df = stock_em_hsgt_south_net_flow_in(
        indicator="沪股通"
    )
    print(stock_em_hsgt_south_net_flow_in_df)

    stock_em_hsgt_south_cash_df = stock_em_hsgt_south_cash(indicator="沪股通")
    print(stock_em_hsgt_south_cash_df)

    stock_em_hsgt_south_acc_flow_in_df = stock_em_hsgt_south_acc_flow_in(
        indicator="沪股通"
    )
    print(stock_em_hsgt_south_acc_flow_in_df)

    stock_em_hsgt_hold_stock_df = stock_em_hsgt_hold_stock(
        market="北向", indicator="今日排行"
    )
    print(stock_em_hsgt_hold_stock_df)

    stock_em_hsgt_hold_stock_df = stock_em_hsgt_hold_stock(
        market="沪股通", indicator="5日排行"
    )
    print(stock_em_hsgt_hold_stock_df)

    stock_em_hsgt_hold_stock_df = stock_em_hsgt_hold_stock(
        market="沪股通", indicator="10日排行"
    )
    print(stock_em_hsgt_hold_stock_df)

    stock_em_hsgt_stock_statistics_df = stock_em_hsgt_stock_statistics(
        symbol="北向持股", start_date="20211102", end_date="20211102"
    )
    print(stock_em_hsgt_stock_statistics_df)

    stock_em_hsgt_stock_statistics_df = stock_em_hsgt_stock_statistics(
        symbol="北向持股", start_date="20210915", end_date="20210916"
    )
    print(stock_em_hsgt_stock_statistics_df)

    stock_em_hsgt_stock_statistics_df = stock_em_hsgt_stock_statistics(
        symbol="沪股通持股", start_date="20210915", end_date="20210916"
    )
    print(stock_em_hsgt_stock_statistics_df)

    stock_em_hsgt_stock_statistics_df = stock_em_hsgt_stock_statistics(
        symbol="深股通持股", start_date="20210915", end_date="20210916"
    )
    print(stock_em_hsgt_stock_statistics_df)

    stock_em_hsgt_institution_statistics_df = stock_em_hsgt_institution_statistics(
        market="北向持股", start_date="20210915", end_date="20210916"
    )
    print(stock_em_hsgt_institution_statistics_df)

    stock_em_hsgt_institution_statistics_df = stock_em_hsgt_institution_statistics(
        market="南向持股", start_date="20210915", end_date="20210916"
    )
    print(stock_em_hsgt_institution_statistics_df)

    stock_em_hsgt_institution_statistics_df = stock_em_hsgt_institution_statistics(
        market="沪股通持股", start_date="20211015", end_date="20211016"
    )
    print(stock_em_hsgt_institution_statistics_df)

    stock_em_hsgt_institution_statistics_df = stock_em_hsgt_institution_statistics(
        market="深股通持股", start_date="20211015", end_date="20211016"
    )
    print(stock_em_hsgt_institution_statistics_df)

    stock_em_hsgt_hist_df = stock_em_hsgt_hist(symbol="港股通沪")
    print(stock_em_hsgt_hist_df)

    stock_em_hsgt_industry_rank_df = stock_em_hsgt_board_rank(symbol="北向资金增持行业板块排行", indicator="1年")
    print(stock_em_hsgt_industry_rank_df)

    stock_hsgt_individual_em_df = stock_hsgt_individual_em(stock="002008")
    print(stock_hsgt_individual_em_df)

    stock_hsgt_individual_detail_em_df = stock_hsgt_individual_detail_em(stock="600596", start_date="20210830", end_date="20211026")
    print(stock_hsgt_individual_detail_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2020/7/12 21:51
Desc: 国泰君安期货-交易日历数据表
https://www.gtjaqh.com/pc/calendar.html
"""
import pandas as pd
import requests


def futures_rule(trade_date: str = "20200712") -> pd.DataFrame:
    """
    国泰君安期货-交易日历数据表
    https://www.gtjaqh.com/pc/calendar.html
    :return: 交易日历数据
    :rtype: pandas.DataFrame
    """
    url = "https://www.gtjaqh.com/fn/128"
    params = {"base_date": f"{trade_date}"}
    r = requests.post(url, json=params)
    temp_df = pd.DataFrame(r.json()["data"])
    temp_df = temp_df[temp_df["tradingday"] == trade_date]
    if not temp_df["events"].values[0]:
        return f"{trade_date} 查询时间过早或者不是交易日"
    else:
        table_df = pd.read_html(temp_df["events"].values[0][0]["content"], header=1)[0]
        table_df.dropna(axis=1, how="all", inplace=True)
    return table_df


if __name__ == '__main__':
    futures_rule_df = futures_rule(trade_date="20210923")
    print(futures_rule_df)



# -*- coding:utf-8 -*-
# /usr/bin/env python
"""
Date: 2020/10/12 18:16
Desc: To test intention, just write test code here!
"""
from mssdk.cost.cost_living import cost_living


def test_cost_living():
    """
    just for test aim
    :return: assert result
    :rtype: assert
    """
    cost_living_df = cost_living()
    assert cost_living_df.shape[0] > 0


if __name__ == "__main__":
    test_cost_living_df = test_cost_living()
    print(test_cost_living_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/5/4 15:12
Desc: 东方财富网-数据中心-研究报告-盈利预测
http://data.eastmoney.com/report/profitforecast.jshtml
"""
from datetime import datetime

import pandas as pd
import requests
from tqdm import tqdm


def stock_profit_forecast():
    """
    东方财富网-数据中心-研究报告-盈利预测
    http://data.eastmoney.com/report/profitforecast.jshtml
    :return: 盈利预测
    :rtype: pandas.DataFrame
    """
    url = "http://reportapi.eastmoney.com/report/predic"
    date_now = datetime.now().date().isoformat()
    date_previous = date_now.replace(date_now[:4], str(int(date_now[:4]) - 2))
    params = {
        "dyCode": "*",
        "pageNo": "1",
        "pageSize": "100",
        "fields": "",
        "beginTime": date_previous,
        "endTime": date_now,
        "hyCode": "*",
        "gnCode": "*",
        "marketCode": "*",
        "sort": "count,desc",
        "p": "1",
        "pageNum": "1",
        "_": "1615374649216",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    page_num = data_json["TotalPage"]
    big_df = pd.DataFrame()
    for page in tqdm(range(1, page_num + 1)):
        params = {
            "dyCode": "*",
            "pageNo": page,
            "pageSize": "100",
            "fields": "",
            "beginTime": date_previous,
            "endTime": date_now,
            "hyCode": "*",
            "gnCode": "*",
            "marketCode": "*",
            "sort": "count,desc",
            "p": page,
            "pageNum": page,
            "_": "1615374649216",
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["data"])
        big_df = big_df.append(temp_df, ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = range(1, len(big_df) + 1)
    big_df.columns = [
        "序号",
        "名称",
        "代码",
        "研报数",
        "机构投资评级(近六个月)-买入",
        "机构投资评级(近六个月)-增持",
        "机构投资评级(近六个月)-中性",
        "机构投资评级(近六个月)-减持",
        "机构投资评级(近六个月)-卖出",
        "_",
        "_",
        "_",
        "_",
        f"{int(date_previous[:4])+2}预测每股收益",
        "_",
        "_",
        f"{int(date_previous[:4])+3}预测每股收益",
        f"{int(date_previous[:4])+4}预测每股收益",
        "_",
        "_",
        "_",
        "_",
        "_",
        f"{int(date_previous[:4])+1}预测每股收益",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]

    big_df = big_df[
        [
            "序号",
            "代码",
            "名称",
            "研报数",
            "机构投资评级(近六个月)-买入",
            "机构投资评级(近六个月)-增持",
            "机构投资评级(近六个月)-中性",
            "机构投资评级(近六个月)-减持",
            "机构投资评级(近六个月)-卖出",
            f"{int(date_previous[:4])+1}预测每股收益",
            f"{int(date_previous[:4])+2}预测每股收益",
            f"{int(date_previous[:4])+3}预测每股收益",
            f"{int(date_previous[:4])+4}预测每股收益",
        ]
    ]
    return big_df


if __name__ == "__main__":
    stock_profit_forecast_df = stock_profit_forecast()
    print(stock_profit_forecast_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/1/26 10:58
Desc: 金融期权数据
http://www.sse.com.cn/assortment/options/price/
"""
import pandas as pd
import requests

from mssdk.option.cons import (
    SH_OPTION_URL_50,
    SH_OPTION_PAYLOAD,
    SH_OPTION_PAYLOAD_OTHER,
    SH_OPTION_URL_KING_50,
    SH_OPTION_URL_300,
    SH_OPTION_URL_KING_300,
    CFFEX_OPTION_URL_300,
)


def option_finance_underlying(symbol: str = "50ETF") -> pd.DataFrame:
    """
    期权标的当日行情, 目前只有 华夏上证50ETF, 华泰柏瑞沪深300ETF 两个产品
    http://www.sse.com.cn/assortment/options/price/
    :param symbol: 50ETF 或 300ETF
    :type symbol: str
    :return: 期权标的当日行情
    :rtype: pandas.DataFrame
    """
    if symbol == "50ETF":
        res = requests.get(SH_OPTION_URL_50, params=SH_OPTION_PAYLOAD)
        data_json = res.json()
        raw_data = pd.DataFrame(data_json["list"])
        raw_data.at[0, 0] = "510050"
        raw_data.at[0, 8] = pd.to_datetime(
            str(data_json["date"]) + str(data_json["time"]), format="%Y%m%d%H%M%S"
        )
        raw_data.columns = [
            "代码",
            "名称",
            "当前价",
            "涨跌",
            "涨跌幅",
            "振幅",
            "成交量(手)",
            "成交额(万元)",
            "更新日期",
        ]
        return raw_data
    else:
        res = requests.get(SH_OPTION_URL_300, params=SH_OPTION_PAYLOAD)
        data_json = res.json()
        raw_data = pd.DataFrame(data_json["list"])
        raw_data.at[0, 0] = "510300"
        raw_data.at[0, 8] = pd.to_datetime(
            str(data_json["date"]) + str(data_json["time"]), format="%Y%m%d%H%M%S"
        )
        raw_data.columns = [
            "代码",
            "名称",
            "当前价",
            "涨跌",
            "涨跌幅",
            "振幅",
            "成交量(手)",
            "成交额(万元)",
            "更新日期",
        ]
        return raw_data


def option_finance_board(symbol: str = "嘉实沪深300ETF期权", end_month: str = "2103") -> pd.DataFrame:
    """
    期权的当日具体的行情数据, 主要为三个: 华夏上证50ETF期权, 华泰柏瑞沪深300ETF期权, 嘉实沪深300ETF期权, 沪深300股指期权
    http://www.sse.com.cn/assortment/options/price/
    http://www.szse.cn/market/product/option/index.html
    http://www.cffex.com.cn/hs300gzqq/
    :param symbol: 华夏上证50ETF期权 or 华泰柏瑞沪深300ETF期权 or 嘉实沪深300ETF期权 or 沪深300股指期权
    :type symbol: str
    :param end_month: 2003; 2020年3月到期的期权
    :type end_month: str
    :return: 当日行情
    :rtype: pandas.DataFrame
    """
    end_month = end_month[-2:]
    if symbol == "华夏上证50ETF期权":
        res = requests.get(
            SH_OPTION_URL_KING_50.format(end_month), params=SH_OPTION_PAYLOAD_OTHER
        )
        data_json = res.json()
        raw_data = pd.DataFrame(data_json["list"])
        raw_data.index = [str(data_json["date"]) + str(data_json["time"])] * data_json[
            "total"
        ]
        raw_data.columns = ["合约交易代码", "当前价", "涨跌幅", "前结价", "行权价"]
        raw_data["数量"] = [data_json["total"]] * data_json["total"]
        return raw_data
    elif symbol == "华泰柏瑞沪深300ETF期权":
        res = requests.get(
            SH_OPTION_URL_KING_300.format(end_month), params=SH_OPTION_PAYLOAD_OTHER
        )
        data_json = res.json()
        raw_data = pd.DataFrame(data_json["list"])
        raw_data.index = [str(data_json["date"]) + str(data_json["time"])] * data_json[
            "total"
        ]
        raw_data.columns = ["合约交易代码", "当前价", "涨跌幅", "前结价", "行权价"]
        raw_data["数量"] = [data_json["total"]] * data_json["total"]
        return raw_data
    elif symbol == "嘉实沪深300ETF期权":
        url = "http://www.szse.cn/api/report/ShowReport/data"
        params = {
            "SHOWTYPE": "JSON",
            "CATALOGID": "ysplbrb",
            "TABKEY": "tab1",
            "PAGENO": "1",
            "random": "0.10642298535346595",
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        page_num = data_json[0]["metadata"]["pagecount"]
        big_df = pd.DataFrame()
        for page in range(1, page_num+1):
            params = {
                "SHOWTYPE": "JSON",
                "CATALOGID": "ysplbrb",
                "TABKEY": "tab1",
                "PAGENO": page,
                "random": "0.10642298535346595",
            }
            r = requests.get(url, params=params)
            data_json = r.json()
            temp_df = pd.DataFrame(data_json[0]["data"])
            big_df = big_df.append(temp_df, ignore_index=True)
        big_df.columns = [
            "合约编码",
            "合约简称",
            "标的名称",
            "类型",
            "行权价",
            "合约单位",
            "期权行权日",
            "行权交收日",
        ]
        big_df["期权行权日"] = pd.to_datetime(big_df["期权行权日"])
        big_df["end_month"] = big_df["期权行权日"].dt.month.astype(str).str.zfill(2)
        big_df = big_df[big_df["end_month"] == end_month]
        del big_df["end_month"]
        return big_df
    elif symbol == "沪深300股指期权":
        raw_df = pd.read_table(CFFEX_OPTION_URL_300, sep=",")
        raw_df["end_month"] = (
            raw_df["instrument"].str.split("-", expand=True).iloc[:, 0].str.slice(4,)
        )
        raw_df = raw_df[raw_df["end_month"] == end_month]
        del raw_df["end_month"]
        return raw_df


if __name__ == "__main__":
    option_finance_underlying_df = option_finance_underlying(symbol="300ETF")
    print(option_finance_underlying_df)
    option_finance_board_df = option_finance_board(symbol="华夏上证50ETF期权", end_month="2003")
    print(option_finance_board_df)
    option_finance_board_df = option_finance_board(symbol="嘉实沪深300ETF期权", end_month="2103")
    print(option_finance_board_df)
    option_finance_board_df = option_finance_board(symbol="华泰柏瑞沪深300ETF期权", end_month="2103")
    print(option_finance_board_df)
    option_finance_board_df = option_finance_board(symbol="沪深300股指期权", end_month="2003")
    print(option_finance_board_df)
